package com.studio.nx.framework.jwt.util;

import cn.hutool.core.util.IdUtil;
import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTCreator;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.studio.nx.framework.jwt.dto.LoginAccountDto;
import com.studio.nx.framework.jwt.exception.JWTException;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import java.io.InputStream;
import java.time.Instant;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.Properties;

public class JWTUtil {

    private static ThreadLocal<LoginAccountDto> localUser = new ThreadLocal();

    public static void setLocalUser(ThreadLocal<LoginAccountDto> localUser) {
        JWTUtil.localUser = localUser;
    }

    public static ThreadLocal<LoginAccountDto> getLocalUser() {
        return localUser;
    }

    /**
     * 加密算法
     */
    private static Algorithm algorithm;

    private static final String ISSUER = "com.studio.nx";
    private static final String SUBJECT = "account";
    public static final Long TOKEN_EXPIRE_SECONDS = 2 * 24 * 60 * 60L; // 秒
    public static final Long TOKEN_EXPIRE_MILLIS = 2 * 24 * 60 * 60 * 1000L; // 毫秒
    public static final Long REFRESH_TOKEN_EXPIRE_TIME = 2 * 24 * 60 * 60 * 1000L;

    private static final String ACCOUNT_ID = "accountId";
    private static final String NICKNAME = "nickname";

    static {
        try {
            InputStream in = JWTUtil.class.getClassLoader().getResourceAsStream("application.yaml");
            Properties props = new Properties();
            props.load(in);
            String appSecret = StringUtils.isEmpty(props.getProperty("appSecret")) ? "wx84ce042499249eb0" : props.getProperty("appSecret");
            algorithm = Algorithm.HMAC512(appSecret);
        } catch (Exception e) {
            throw new RuntimeException("load config error " + e.getMessage());
        }
    }

    /**
     * 根据账号登陆信息创建 token
     *
     * @param accountId 账号标识
     * @return 返回 jwt token
     */
    public static String createJWT(String accountId) {
        Date now = Date.from(Instant.now());
        String jwtId = IdUtil.randomUUID();
        JWTCreator.Builder builder = JWT.create()
                .withIssuer(ISSUER) // 用于说明该JWT是由谁签发的
                .withSubject(SUBJECT) // 用于说明该JWT面向的对象
//                    .withAudience() // 用于说明该JWT发送给的用户
                .withExpiresAt(Date.from(ZonedDateTime.now().plusMinutes(TOKEN_EXPIRE_MILLIS).toInstant())) // 数字类型，说明该JWT过期的时间
                .withNotBefore(now) // 生效时间
                .withIssuedAt(now) // 签发时间
                .withJWTId(jwtId); // 说明标明JWT的唯一ID

        // 自定义字段存储业务数据
        builder.withClaim(ACCOUNT_ID, accountId);
        builder.withClaim(NICKNAME, "");

        return builder.sign(algorithm);
    }

    /**
     * 功能描述：解析验证token
     *
     * @param token 加密后的token字符串
     * @return
     */
    public static void verify(String token) {
        if (StringUtils.isEmpty(token)) {
            throw new JWTException("缺失token");
        }

        try {
            JWTVerifier verifier = JWT.require(algorithm)
                    .withIssuer(ISSUER)
                    .withSubject(SUBJECT)
                    .acceptExpiresAt(System.currentTimeMillis() + TOKEN_EXPIRE_SECONDS) //JWT 正确的配置续期姿势
                    .build();

            DecodedJWT jwt = verifier.verify(token);

            LoginAccountDto loginAccountDto = new LoginAccountDto();
            loginAccountDto.setAccountId(jwt.getClaim(ACCOUNT_ID).asString());
            loginAccountDto.setNickname(jwt.getClaim(NICKNAME).asString());
            localUser.set(loginAccountDto);
        } catch (Exception e) {
            throw new JWTException("解密token失败");
        }
    }

    /**
     * 功能描述：
     * todo 暂时先放这里
     *
     * @return
     */
    public static String getToken() {
        String token = "";
        RequestAttributes ra = RequestContextHolder.getRequestAttributes();
        ServletRequestAttributes sra = (ServletRequestAttributes) ra;
        HttpServletRequest request = sra.getRequest();

        // 1. 尝试从 参数/head 中获取身份token
        if (StringUtils.isEmpty(token)) {
            token = request.getParameter("token");
            if (StringUtils.isEmpty(token)) {
                token = request.getHeader("token");
            }
        }

        // 2. 尝试从 参数/head 中获取身份Authorization
        if (StringUtils.isEmpty(token)) {
            token = request.getParameter("Authorization");
            if (StringUtils.isEmpty(token)) {
                token = request.getHeader("Authorization");
            }
        }

        // 3. 尝试从 参数/head 中获取身份authorization
        if (StringUtils.isEmpty(token)) {
            token = request.getParameter("authorization");
            if (StringUtils.isEmpty(token)) {
                token = request.getHeader("authorization");
            }
        }

        // 3. 尝试从Cookie中获取身份token/Authorization
        if (StringUtils.isEmpty(token)) {
            Cookie[] cookies = request.getCookies();
            if (cookies != null && cookies.length > 0) {
                for (Cookie cookie : cookies) {
                    if (cookie.getName().equals("token") || cookie.getName().equals("Authorization")) {
                        token = cookie.getValue();
                        break;
                    }
                }
            }
        }

        return token;
    }

}
