package com.studio.nx.framework.aspect;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.framework.jwt.util.JWTUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.util.StopWatch;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.support.StandardMultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 * 功能描述：访问日志AOP拦截
 *
 * @Author 梁展鹏
 * @Date 2020/5/24 13:13
 */
@Slf4j
@Aspect
@Component
public class AccessLogAspect {

    private final static String ERROR_FORMAT = "api调用出错:%s, 方法%s,参数%s,异常%s";
    private final static ObjectMapper mapper = new ObjectMapper();

    @Pointcut("execution(* com.studio.nx.*.controller.*.*Controller.*(..)))")
    public void accessLog() {
    }

    @AfterReturning(value = "accessLog()", argNames = "point, retVl", returning = "retVl")
    public void doAfterReturning(JoinPoint point, Object retVl) {
        RequestAttributes ra = RequestContextHolder.getRequestAttributes();
        ServletRequestAttributes sra = (ServletRequestAttributes) ra;
        HttpServletRequest request = sra.getRequest();

        log.info("方法名：{}.{}, 请求头：{}, token：{}, param：{}", point.getTarget().getClass().getName(), this.getMethod(point).getName(), this.getHeadersInfo(request), JWTUtil.getToken(), this.getParamInfo(request));

        try {
            Object[] params = point.getArgs();
            for (Object object : params) {
//                if (object instanceof String || object instanceof BaseRequest || object instanceof BasePageRequest) {
//                    log.info("请求参数：[{}]", mapper.writeValueAsString(object));
//                }
            }
//            log.info("返回参数：[{}]", mapper.writeValueAsString(retVl));
        } catch (Exception e) {
            log.error("内部异常：[{}], [{}]", e.getMessage(), e);
        }
    }

    @Around(value = "accessLog()")
    public Object doAround(ProceedingJoinPoint joinPoint) throws Throwable {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();
        HttpServletResponse response = attributes.getResponse();

        // 拿到ip地址、请求路径、token
        String url = request.getRequestURL().toString();
        String ip = request.getRemoteAddr(); // todo IpUtils.getIpAdrress(request);
        Object[] params = joinPoint.getArgs();
        String token = JWTUtil.getToken();

        // 从切面织入点处通过反射机制获取织入点处的方法
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        // 获取切入点所在的方法
        Method method = signature.getMethod();

        /* 获取类名称 */
        Api api = method.getDeclaringClass().getAnnotation(Api.class);
        String[] classModuleName = {};
        if (null != api) {
            classModuleName = api.tags();
        }

        /* 获取方法名称 */
        ApiOperation apiOperation = method.getAnnotation(ApiOperation.class);
        String methodName = ""; // 方法名称
        if (null != apiOperation) {
            methodName = apiOperation.value();
        }

        /* 记录执行时间 */
        StopWatch watch = new StopWatch();
        watch.start();
        try {
            Object result = joinPoint.proceed(joinPoint.getArgs());
            return result;
        } catch (BusinessException ex) {
            log.error("错误：[{}], [{}]", ex.getMessage(), ex);

            return ResponseEntity.ok(BaseResponse.failure(HttpStatus.INTERNAL_SERVER_ERROR.value(), ex.getMessage()));
        } catch (Exception ex) {
            log.error("错误：[{}], [{}]", ex.getMessage(), ex);

            return ResponseEntity.ok(BaseResponse.failure(HttpStatus.INTERNAL_SERVER_ERROR.value(), "服务异常，请联系管理员"));
        } finally {
            watch.stop();

            log.info("Swagger UI：{}-{}, 请求地址：{}, 客户端ip：{}, 时间：{}ms", classModuleName, methodName, url, ip, watch.getTotalTimeMillis());
        }

        // 插入系统日志表
//        SysLog sysLog = new SysLog();
//        sysLog.setName(methodName);
//        sysLog.setUrl(url);
//        sysLog.setIp(ip);
//        // 获取用户信息
//        if (token == null) {
//            // 非法人员
//            sysLog.setUserId(0);
////            sysLog.setName(result.getMessage());
//        } else {
//            if (userMapper.getUserInfoByToken(token) != null) {
//                sysLog.setUserId(userMapper.getUserInfoByToken(token).getId());
//            }
//        }
//        sysLog.setStatus(result.getCode());
//        sysLog.setExecuteTime(execTime + " ms");
//        sysLog.insert();
    }

    private Method getMethod(JoinPoint pjp) {
        return ((MethodSignature) pjp.getSignature()).getMethod();
    }

    private Map<String, String> getHeadersInfo(HttpServletRequest request) {
        Map<String, String> map = new HashMap();
        Enumeration<?> headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String key = (String) headerNames.nextElement();
            String value = request.getHeader(key);
            map.put(key, value);
        }
        return map;
    }

    private Map<String, String> getParamInfo(HttpServletRequest request) {
        Map<String, String> map = new HashMap();
        Enumeration<?> paramsNames = request.getParameterNames();
        while (paramsNames.hasMoreElements()) {
            String key = (String) paramsNames.nextElement();
            String value = request.getParameter(key);
            map.put(key, value);
        }
        return map;
    }

    private void sendErrorMessage(Exception e, String className, String methodName, Object params) {
        /* 判断请求是MultipartHttpServletRequest（上传文件） */
        if (params instanceof StandardMultipartHttpServletRequest) {
            params = null;
        }
        StringWriter sw = new StringWriter();
        e.printStackTrace(new PrintWriter(sw, true));
        String str = sw.toString();
        String content;
        try {
            content = String.format(ERROR_FORMAT, className, methodName, mapper.writeValueAsString(params), str);
            log.error("访问接口失败 : {}", content);
        } catch (JsonProcessingException e1) {
            e1.printStackTrace();
        }
    }

}
