package com.studio.nx.framework.util.file;

import java.io.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

public class ZipUtil {

    /**
     * 功能描述：压缩文件
     *
     * @param filePath 待压缩的文件路径
     * @param zipName  压缩包文件名
     * @param comment  压缩包备注
     * @return 压缩后的文件
     */
    public static File zip(String filePath, String zipName, String comment) throws IOException {
        File target = null;
        File source = new File(filePath);
        if (source.exists()) {
            target = new File(source.getParent(), zipName); // 在父目录创建zip，因为原目录有可能打完zip后会被删除
            if (target.exists()) {
                target.delete(); // 删除旧的文件
            }

            FileOutputStream fos = null;
            ZipOutputStream zos = null;
            try {
                fos = new FileOutputStream(target);
                zos = new ZipOutputStream(new BufferedOutputStream(fos));

                /* 添加对应的文件Entry */
                addEntry("/", source, zos);

                /* 增加压缩包备注 */
                zos.setComment(comment);
            } catch (IOException e) {
                throw new RuntimeException(e);
            } finally {
                if (zos != null) {
                    zos.close();
                }

                if (fos != null) {
                    fos.close();
                }
            }
        }

        return target;
    }

    /**
     * 功能描述：扫描添加文件Entry
     *
     * @param base   基路径
     * @param source 源文件
     * @param zos    Zip文件输出流
     * @throws IOException
     */
    private static void addEntry(String base, File source, ZipOutputStream zos) throws IOException {
        // 按目录分级，形如：/aaa/bbb.txt
        String entry = base + source.getName();
        if (source.isDirectory()) {
            for (File file : source.listFiles()) {
                // 递归列出目录下的所有文件，添加文件Entry
                addEntry(entry + File.separator, file, zos);
            }
        } else {
            FileInputStream fis = null;
            BufferedInputStream bis = null;
            try {
                byte[] buffer = new byte[1024 * 10];
                fis = new FileInputStream(source);
                bis = new BufferedInputStream(fis, buffer.length);
                int read = 0;
                zos.putNextEntry(new ZipEntry(entry));
                while ((read = bis.read(buffer, 0, buffer.length)) != -1) {
                    zos.write(buffer, 0, read);
                }
                zos.closeEntry();
            } finally {
                if (bis != null) {
                    bis.close();
                }

                if (fis != null) {
                    fis.close();
                }
            }
        }
    }

    /**
     * 功能描述：解压文件
     *
     * @param filePath 压缩文件路径
     */
    public static void unzip(String filePath) throws IOException {
        File source = new File(filePath);
        if (source.exists()) {
            ZipInputStream zis = null;
            BufferedOutputStream bos = null;
            try {
                zis = new ZipInputStream(new FileInputStream(source));
                ZipEntry entry = null;
                while ((entry = zis.getNextEntry()) != null
                        && !entry.isDirectory()) {
                    File target = new File(source.getParent(), entry.getName());
                    if (!target.getParentFile().exists()) {
                        // 创建文件父目录
                        target.getParentFile().mkdirs();
                    }
                    // 写入文件
                    bos = new BufferedOutputStream(new FileOutputStream(target));
                    int read = 0;
                    byte[] buffer = new byte[1024 * 10];
                    while ((read = zis.read(buffer, 0, buffer.length)) != -1) {
                        bos.write(buffer, 0, read);
                    }
                    bos.flush();
                }
                zis.closeEntry();
            } catch (IOException e) {
                throw new RuntimeException(e);
            } finally {
                if (bos != null) {
                    bos.close();
                }

                if (zis != null) {
                    zis.close();
                }
            }
        }
    }

}
