package com.studio.nx.uiac.controller;

import cn.hutool.core.date.DateUtil;
import com.studio.nx.framework.annotation.OperationLog;
import com.studio.nx.framework.dto.req.BasePageRequest;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.uiac.controller.response.AccountAdminCountResponse;
import com.studio.nx.uiac.controller.response.AccountAdminResponse;
import com.studio.nx.uiac.controller.search.AccountSearchDto;
import com.studio.nx.uiac.dto.AccountDto;
import com.studio.nx.uiac.mapper.AccountMapper;
import com.studio.nx.uiac.service.AccountRoleService;
import com.studio.nx.uiac.service.AccountService;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.Date;
import java.util.List;

@Api(tags = "账号管理")
@Slf4j
@RestController("FrameworkAccountController")
@RequestMapping("/framework/uiac/account")
public class AccountController {

    @Autowired
    private AccountMapper mapper;

    @Autowired
    private AccountService service;

    @Autowired
    private AccountRoleService accountRoleService;

    @ApiOperation(value = "查询[账号]列表")
    @PostMapping("/page")
    public ResponseEntity<BaseResponse<BasePageResponse<AccountAdminResponse>>> page(@Valid @RequestBody BasePageRequest<AccountSearchDto> request) {
        return ResponseEntity.ok(BaseResponse.ok(service.adminPage(request.getData(), request.getPage(), request.getLimit())));
    }

    @ApiOperation(value = "查询[账号]列表统计")
    @PostMapping("/count")
    public ResponseEntity<BaseResponse<AccountAdminCountResponse>> count(@Valid @RequestBody BasePageRequest<AccountSearchDto> request) {

        AccountSearchDto search = request.getData();

        AccountAdminCountResponse response = new AccountAdminCountResponse();
        search.setStartTime(null);
        search.setEndTime(null);
        response.setAll(mapper.count(search));

        search.setStartTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.firstDayOfMonth())), LocalTime.MIN));
        search.setEndTime(LocalDateTime.of(LocalDate.from(LocalDateTime.now().with(TemporalAdjusters.lastDayOfMonth())), LocalTime.MAX));
        response.setMonth(mapper.count(search));

        search.setStartTime(DateUtil.toLocalDateTime(DateUtil.beginOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        search.setEndTime(DateUtil.toLocalDateTime(DateUtil.endOfWeek(new Date(), Boolean.FALSE).toJdkDate()));
        response.setWeek(mapper.count(search));

        // todo 区分账号类型
        return ResponseEntity.ok(BaseResponse.ok(response));
    }

    @ApiOperation(value = "查询[账号]下拉选择")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "keyword", value = "关键词", required = true, paramType = "query", defaultValue = "")
    })
    @GetMapping("/search-select")
    public ResponseEntity<BaseResponse<List<AccountAdminResponse>>> searchSelect(@Valid @NotBlank(message = "关键词") String keyword) {
        if (!StringUtils.hasText(keyword)) {
            ResponseEntity.ok(BaseResponse.ok());
        }

        return ResponseEntity.ok(BaseResponse.ok(service.searchSelect(keyword)));
    }

    @OperationLog("新增 - 账号")
    @ApiOperation(value = "新增[账号]")
    @PostMapping("/save")
    public ResponseEntity<BaseResponse<?>> save(@Valid @RequestBody BaseRequest<AccountDto> request) {
        service.save(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("更新 - 账号")
    @ApiOperation(value = "更新[账号]")
    @PostMapping("/update")
    public ResponseEntity<BaseResponse<?>> update(@Valid @RequestBody BaseRequest<AccountDto> request) {
        service.update(request.getData());
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("替换删除状态 - 账号")
    @ApiOperation(value = "替换[账号]删除状态")
    @DeleteMapping("/del/{id:\\d+}")
    public ResponseEntity<BaseResponse<?>> handleIsDel(@Valid @ApiParam("标识") @PathVariable String id) {
        service.handleIsDel(id);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("授权账号角色")
    @ApiOperation(value = "授权账号角色")
    @PostMapping("/{accountId:\\d+}/save-account-role/{roleId:\\d+}")
    public ResponseEntity<BaseResponse<?>> saveAccountRole(@Valid @ApiParam("账号标识") @PathVariable String accountId, @Valid @ApiParam("角色标识") @PathVariable String roleId) {
        accountRoleService.save(accountId, roleId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

    @OperationLog("取消授权账号角色")
    @ApiOperation(value = "取消授权账号角色")
    @DeleteMapping("/{accountId:\\d+}/del-account-role/{roleId:\\d+}")
    public ResponseEntity<BaseResponse<?>> delAccountRole(@Valid @ApiParam("账号标识") @PathVariable String accountId, @Valid @ApiParam("角色标识") @PathVariable String roleId) {
        accountRoleService.delete(accountId, roleId);
        return ResponseEntity.ok(BaseResponse.ok());
    }

}
