package com.studio.nx.uiac.service;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.json.JSONObject;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.framework.error.exception.NxFrameworkException;
import com.studio.nx.notice.service.sms.AliDaYuService;
import com.studio.nx.oss.local.config.LocalYaml;
import com.studio.nx.uiac.constant.ImgCodeEnum;
import com.studio.nx.uiac.constant.RedisKeyConstant;
import com.studio.nx.uiac.controller.request.auth.MobileNumberSmsCodeRequest;
import com.studio.nx.uiac.controller.request.auth.MobileNumberSmsSignUpRequest;
import com.studio.nx.uiac.controller.response.SignInResponse;
import com.studio.nx.uiac.entity.Account;
import com.studio.nx.uiac.entity.AccountAuths;
import com.studio.nx.uiac.mapper.AccountAuthsMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.transaction.Transactional;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 功能描述：手机号码 + 短信验证码
 *
 * @Author 梁展鹏
 * @Date 2021/6/7 15:35
 */
@Slf4j
@Service
public class AuthMobileNumberSmsService {

    @Value("${spring.profiles.active}")
    private String active;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private LocalYaml localYaml;

    @Autowired
    private AliDaYuService aliDaYuService;

    @Autowired
    private AuthService authService;

    @Autowired
    private AccountService accountService;

    @Autowired
    private AccountAuthsService accountAuthsService;

    @Autowired
    private AccountAuthsMapper accountAuthsMapper;

    @Autowired
    private AccountInviteService accountInviteService;

    /**
     * 功能描述：发送短信验证码
     * 1. 判断图片验证码是否正确（防刷）
     * 2. 随机生成验证码并发送到短信网关服务
     *
     * @param request
     */
    public void sendMobileNumberSMSCode(MobileNumberSmsCodeRequest request) {
        String uuid = request.getUuid();
        String code = request.getCode();
        String mobileNumber = request.getMobileNumber();
        authService.checkImgCode(ImgCodeEnum.MobileNumberSms, uuid, code);

        // todo 有需要考虑手机号码黑名单防刷短信

        /* 随机生成6位数字验证码 */
        String smsCode = "000000"; // 方便开发。暂定短信验证码为6个0
        if ("prod".equals(active)) {
            smsCode = RandomUtil.randomNumbers(6);
        }

        // 存入redis并设置过期时间为70秒
        stringRedisTemplate.opsForValue().set(RedisKeyConstant.PlatformMobileNumberSMSCode(mobileNumber), smsCode, 70, TimeUnit.SECONDS);

        /* 调用短信网关 */
        JSONObject templateParam = new JSONObject();
        templateParam.putOnce("code", smsCode);

        /* 如果该手机号码是白名单，则不发送短信验证码 */
        if (StringUtils.hasText(authService.getWhiteList().get(mobileNumber))) {
            return;
        }

        if ("prod".equals(active)) { // todo 待正式上线后可重新申请短信模版，需要追加有效性的字眼描述
            aliDaYuService.sendSMS(mobileNumber, "幼享乐", "SMS_218675073", templateParam.toString());
        }
    }

    /**
     * 功能描述：手机号码 + 短信验证码 = 注册
     * 1. 校验手机号啊、短信验证码是否正确
     * 2. 判断手机号码是否存在，
     * 3. 新增手机号码账号数据
     * 4. 判断是否存在邀请记录
     * 5. 返回账号登录成功到token
     *
     * @param request
     * @return
     */
    @Transactional
    public SignInResponse mobileNumberSMSSignUp(MobileNumberSmsSignUpRequest request, HttpServletRequest _request) {
        /* 校验短信验证码 */
        String mobileNumber = request.getMobileNumber();
        String smsCode = request.getSmsCode();
        String thirdAccountId = request.getAccountId(); // 第三方账号标识（代表需要绑定账号）

        if (StringUtils.hasText(authService.getWhiteList().get(mobileNumber))) {
            String code = authService.getWhiteList().get(mobileNumber);
            Assert.isTrue(code.equalsIgnoreCase(request.getSmsCode().trim()), "验证码错误");
        } else {
            checkMobileNumberSMSCode(mobileNumber, smsCode);
        }

        /* 注册的手机号码已经存在 */
        Optional<AccountAuths> accountAuthsMobileNumberOptional = accountAuthsService.findByMobileNumber(mobileNumber);
        if (accountAuthsMobileNumberOptional.isPresent()) {
            if (StringUtils.hasText(thirdAccountId)) { // 存在是第三方登录关联
                /* 判断手机号码账号是否有绑定过微信APP */
                Optional<AccountAuths> accountAuthsWeixinOpenAppOpenOptional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN, accountAuthsMobileNumberOptional.get().getAccountId());
                if (accountAuthsWeixinOpenAppOpenOptional.isPresent()) { // 手机号码账号已绑定过微信，不能再绑定
                    throw new BusinessException(String.format("%s已经绑定过微信，不能重复绑定", mobileNumber));
                } else {
                    /* 重新把微信登录绑定到已有手机号码账号下 */
                    Optional<AccountAuths> accountAuthsThirdAccountIdWeixinOpenAppOpenOptional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN, thirdAccountId);
                    if (accountAuthsThirdAccountIdWeixinOpenAppOpenOptional.isPresent()) {
                        AccountAuths thirdAccountIdWeixinOpenAppOpen = accountAuthsThirdAccountIdWeixinOpenAppOpenOptional.get();
                        thirdAccountIdWeixinOpenAppOpen.setAccountId(accountAuthsMobileNumberOptional.get().getAccountId());
                        accountAuthsMapper.updateById(thirdAccountIdWeixinOpenAppOpen);
                    }
                }
            }

            Optional<Account> accountOptional = accountService.findById(accountAuthsMobileNumberOptional.get().getAccountId());
            if (!accountOptional.isPresent()) {
                throw new NxFrameworkException("账号数据异常");
            }

            /* 生成访问令牌 */
            Account _account = accountOptional.get();
            return authService.createSignInResponse(_account.getId(), _account.getNickname(), _account.getAvatarUrl(), _request);
        }

        /* 注册的手机号码首次入库 */
        Account _account = new Account();
        if (StringUtils.hasText(thirdAccountId)) {
            Optional<Account> accountOptional = accountService.findById(thirdAccountId);
            if (!accountOptional.isPresent()) {
                throw new NxFrameworkException("账号数据异常");
            }

            _account = accountOptional.get();
        } else {
            /* 新增账号表数据 */
            _account.setNickname(mobileNumber);
            _account.setRealname("");
            _account.setAvatarUrl(String.format("%s/static/yxl/default-avatar-url.png", localYaml.getHost()));
            _account.setGender(0);
            _account.setType(Account.Type.GENERAL_ACCOUNT);
            accountService.save(_account);

            /* 通过队列实现注册奖励积分 */
            stringRedisTemplate.opsForList().leftPush("yxl:integral-cash:integral_to_sign_up", _account.getId());
        }

        accountAuthsService.save(_account.getId(), AccountAuths.IdentityType.MOBILE_NUMBER, mobileNumber, "", "");

        /* 保存邀请记录 */
        Optional<Account> accountInvitationCodeOptional = accountService.findById(request.getInvitationCode());
        if (StringUtils.hasText(request.getInvitationCode()) && Optional.ofNullable(accountInvitationCodeOptional).isPresent()) {
            accountInviteService.save(request.getInvitationCode(), _account.getId(), _account.getNickname());
        }

        return authService.createSignInResponse(_account.getId(), _account.getNickname(), _account.getAvatarUrl(), _request);
    }

    /**
     * 功能描述：手机号码 + 短信验证码 = 注册
     * 1. 校验手机号啊、短信验证码是否正确
     * 2. 判断手机号码是否存在，
     * 3. 新增手机号码账号数据
     * 4. 判断是否存在邀请记录
     *
     * @param request
     * @return
     */
    @Transactional
    public void h5MobileNumberSMSSignUp(MobileNumberSmsSignUpRequest request, HttpServletRequest _request) {
        /* 校验短信验证码 */
        String mobileNumber = request.getMobileNumber();
        String smsCode = request.getSmsCode();
        String thirdAccountId = request.getAccountId(); // 第三方账号标识（代表需要绑定账号）

        if (StringUtils.hasText(authService.getWhiteList().get(mobileNumber))) {
            String code = authService.getWhiteList().get(mobileNumber);
            Assert.isTrue(code.equalsIgnoreCase(request.getSmsCode().trim()), "验证码错误");
        } else {
            checkMobileNumberSMSCode(mobileNumber, smsCode);
        }

        /* 注册的手机号码已经存在 */
        Optional<AccountAuths> accountAuthsMobileNumberOptional = accountAuthsService.findByMobileNumber(mobileNumber);
        if (accountAuthsMobileNumberOptional.isPresent()) {
            if (StringUtils.hasText(thirdAccountId)) { // 存在是第三方登录关联
                /* 判断手机号码账号是否有绑定过微信APP */
                Optional<AccountAuths> accountAuthsWeixinOpenAppOpenOptional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN, accountAuthsMobileNumberOptional.get().getAccountId());
                if (accountAuthsWeixinOpenAppOpenOptional.isPresent()) { // 手机号码账号已绑定过微信，不能再绑定
                    throw new BusinessException(String.format("%s已经绑定过微信，不能重复绑定", mobileNumber));
                } else {
                    /* 重新把微信登录绑定到已有手机号码账号下 */
                    Optional<AccountAuths> accountAuthsThirdAccountIdWeixinOpenAppOpenOptional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN, thirdAccountId);
                    if (accountAuthsThirdAccountIdWeixinOpenAppOpenOptional.isPresent()) {
                        AccountAuths thirdAccountIdWeixinOpenAppOpen = accountAuthsThirdAccountIdWeixinOpenAppOpenOptional.get();
                        thirdAccountIdWeixinOpenAppOpen.setAccountId(accountAuthsMobileNumberOptional.get().getAccountId());
                        accountAuthsMapper.updateById(thirdAccountIdWeixinOpenAppOpen);
                    }
                }
            }

            Optional<Account> accountOptional = accountService.findById(accountAuthsMobileNumberOptional.get().getAccountId());
            if (!accountOptional.isPresent()) {
                throw new NxFrameworkException("账号数据异常");
            }

            /* 生成访问令牌 */
            Account _account = accountOptional.get();
//            return authService.createSignInResponse(_account.getId(), _account.getNickname(), _account.getAvatarUrl(), _request);
        }

        /* 注册的手机号码首次入库 */
        Account _account = new Account();
        if (StringUtils.hasText(thirdAccountId)) {
            Optional<Account> accountOptional = accountService.findById(thirdAccountId);
            if (!accountOptional.isPresent()) {
                throw new NxFrameworkException("账号数据异常");
            }

            _account = accountOptional.get();
        } else {
            /* 新增账号表数据 */
            _account.setNickname(mobileNumber);
            _account.setRealname("");
            _account.setAvatarUrl(String.format("%s/static/yxl/default-avatar-url.png", localYaml.getHost()));
            _account.setGender(0);
            _account.setType(Account.Type.GENERAL_ACCOUNT);
            accountService.save(_account);

            /* 通过队列实现注册奖励积分 */
            stringRedisTemplate.opsForList().leftPush("yxl:integral-cash:integral_to_sign_up", _account.getId());
        }

        accountAuthsService.save(_account.getId(), AccountAuths.IdentityType.MOBILE_NUMBER, mobileNumber, "", "");

        /* 保存邀请记录 */
        Optional<Account> accountInvitationCodeOptional = accountService.findById(request.getInvitationCode());
        if (StringUtils.hasText(request.getInvitationCode()) && Optional.ofNullable(accountInvitationCodeOptional).isPresent()) {
            accountInviteService.save(request.getInvitationCode(), _account.getId(), _account.getNickname());
        }

//        return authService.createSignInResponse(_account.getId(), _account.getNickname(), _account.getAvatarUrl(), _request);
    }

    public void checkMobileNumberSMSCode(String mobileNumber, String smsCode) {
        /* 校验验证码是否有效 */
        String verCode = stringRedisTemplate.opsForValue().get(RedisKeyConstant.PlatformMobileNumberSMSCode(mobileNumber));

        Assert.hasText(verCode, "验证码不存在或已过期");
        Assert.isTrue(verCode.equalsIgnoreCase(smsCode.trim()), "验证码错误");

        /* 验证码通过后才手动删除 */
        stringRedisTemplate.delete(RedisKeyConstant.PlatformMobileNumberSMSCode(mobileNumber));
    }

}
