package com.studio.nx.uiac.service;

import cn.hutool.core.lang.Assert;
import com.studio.nx.framework.error.exception.NxFrameworkException;
import com.studio.nx.uiac.config.weixin.WxOpenAppYml;
import com.studio.nx.uiac.controller.request.auth.Oauth2WeixinRequest;
import com.studio.nx.uiac.controller.response.SignInResponse;
import com.studio.nx.uiac.dto.weixin.WeixinAccessTokenResponse;
import com.studio.nx.uiac.dto.weixin.WeixinUserInfoResponse;
import com.studio.nx.uiac.entity.Account;
import com.studio.nx.uiac.entity.AccountAuths;
import com.studio.nx.uiac.feign.WeixinFeign;
import com.studio.nx.uiac.mapper.AccountMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.util.Optional;

@Service
public class AuthOauth2Service {

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private WxOpenAppYml wxOpenAppYml;

    @Autowired
    private WeixinFeign weixinFeign;

    @Autowired
    private AuthService authService;

    @Autowired
    private AccountMapper accountMapper;

    @Autowired
    private AccountAuthsService accountAuthsService;

    /**
     * 功能描述：微信登录
     *
     * @param request
     * @param _request
     * @return
     */
    public SignInResponse bindingWeixin(Oauth2WeixinRequest request, HttpServletRequest _request) {
        String thirdAccountId = request.getAccountId();
        Integer type = request.getType();
        /**
         * 判断账号是否已经绑定了微信第三方
         * 如绑定就抛出异常
         */
        if (StringUtils.hasText(thirdAccountId)) {
            checkAccount(type, thirdAccountId);
        }

        /* 校验【微信】签名 todo 微信开放平台下不同类型是否一致接口？？？ */
        WeixinAccessTokenResponse accessTokenResponse = weixinFeign.accessToken(wxOpenAppYml.getAppid(), wxOpenAppYml.getSecret(), request.getCode());
        if (null != accessTokenResponse.getErrcode() || StringUtils.hasText(accessTokenResponse.getErrmsg())) {
            throw new NxFrameworkException("微信绑定失败");
        }

        /* 获取【微信账号】信息 todo 微信开放平台下不同类型是否一致接口？？？ */
        WeixinUserInfoResponse userInfoResponse = weixinFeign.userinfo(accessTokenResponse.getAccessToken(), accessTokenResponse.getOpenid());

        Account _account = null;
        if (StringUtils.hasText(thirdAccountId) && !accountAuthsService.findByIdentifier(AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN, userInfoResponse.getOpenid()).isPresent()) { // 手机号码已注册进来绑定微信的
            _account = bind(type, thirdAccountId, userInfoResponse);
        } else if (accountAuthsService.findByIdentifier(AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN, userInfoResponse.getOpenid()).isPresent()) { // openid账号已存在（单纯微信登录）
            String weixinOpenAppOpenAccountId = accountAuthsService.findByIdentifier(AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN, userInfoResponse.getOpenid()).get().getAccountId();
            _account = accountMapper.selectById(weixinOpenAppOpenAccountId);
            if (!StringUtils.hasText(_account.getNickname())) {
                _account.setNickname(userInfoResponse.getNickname());
            }
            if (!StringUtils.hasText(_account.getAvatarUrl())) {
                _account.setAvatarUrl(userInfoResponse.getHeadimgurl());
            }
            if (null == _account.getGender()) {
                _account.setGender(userInfoResponse.getSex());
            }

            accountMapper.updateById(_account);
        } else { // 首次直接使用微信第三方登录
            _account = saveAccount(type, userInfoResponse);
        }

        return authService.createSignInResponse(_account.getId(), _account.getNickname(), _account.getAvatarUrl(), _request);
    }

    /**
     * 功能描述：校验账号id是否存在已经绑定
     *
     * @param type
     * @param accountId
     */
    private void checkAccount(Integer type, String accountId) {
        Optional<AccountAuths> optional = Optional.empty();
        switch (type) {
            case 0: // 移动应用
                optional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN, accountId);
                break;
            case 1: // 网站应用
                optional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.WEIXIN_OPEN_WEB_OPEN, accountId);
                break;
            case 2: // 公众帐号
                optional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.WEIXIN_OPEN_PUBLIC_OPEN, accountId);
                break;
            case 3: // 微信小程序
                optional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.WEIXIN_OPEN_MINI_PROGRAM_OPENID, accountId);
                break;
            default:
                throw new NxFrameworkException("微信类型出错");
        }

        Assert.isFalse(optional.isPresent(), "目前已绑定微信第三方账号。");
    }

    private Account bind(Integer type, String accountId, WeixinUserInfoResponse userInfoResponse) {
        Account _account = accountMapper.selectById(accountId);
        if (!StringUtils.hasText(_account.getNickname())) {
            _account.setNickname(userInfoResponse.getNickname());
        }
        if (!StringUtils.hasText(_account.getAvatarUrl())) {
            _account.setAvatarUrl(userInfoResponse.getHeadimgurl());
        }
        if (null == _account.getGender()) {
            _account.setGender(userInfoResponse.getSex());
        }
        accountMapper.updateById(_account);

        /* 账号绑定微信openid */
        AccountAuths.IdentityType identityType = null;
        switch (type) {
            case 0: // 移动应用
                identityType = AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN;
                break;
            case 1: // 网站应用
                identityType = AccountAuths.IdentityType.WEIXIN_OPEN_WEB_OPEN;
                break;
            case 2: // 公众帐号
                identityType = AccountAuths.IdentityType.WEIXIN_OPEN_PUBLIC_OPEN;
                break;
            case 3: // 微信小程序
                identityType = AccountAuths.IdentityType.WEIXIN_OPEN_MINI_PROGRAM_OPENID;
                break;
        }
        accountAuthsService.save(_account.getId(), identityType, userInfoResponse.getOpenid(), "", "");

        /* 账号绑定微信unionid */
        if (StringUtils.hasText(userInfoResponse.getUnionid())) {
            accountAuthsService.save(_account.getId(), AccountAuths.IdentityType.WEIXIN_OPEN_UNIONID, userInfoResponse.getUnionid(), "", "");
        }

        return _account;
    }

    private Account saveAccount(Integer type, WeixinUserInfoResponse userInfoResponse) {
        Account _account = new Account();
        _account.setNickname(userInfoResponse.getNickname());
        _account.setRealname("");
        _account.setAvatarUrl(userInfoResponse.getHeadimgurl());
        _account.setGender(userInfoResponse.getSex());
        _account.setType(Account.Type.GENERAL_ACCOUNT);
        _account.setPasswordFailureDateTime(LocalDateTime.now());
        _account.setEndDateTime(LocalDateTime.now());
        _account.setIsLocked(Boolean.FALSE);
        accountMapper.insert(_account);

        /* 账号绑定微信openid */
        AccountAuths.IdentityType identityType = null;
        switch (type) {
            case 0: // 移动应用
                identityType = AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN;
                break;
            case 1: // 网站应用
                identityType = AccountAuths.IdentityType.WEIXIN_OPEN_WEB_OPEN;
                break;
            case 2: // 公众帐号
                identityType = AccountAuths.IdentityType.WEIXIN_OPEN_PUBLIC_OPEN;
                break;
            case 3: // 微信小程序
                identityType = AccountAuths.IdentityType.WEIXIN_OPEN_MINI_PROGRAM_OPENID;
                break;
        }
        accountAuthsService.save(_account.getId(), identityType, userInfoResponse.getOpenid(), "", "");

        /* 账号绑定微信unionid */
        if (StringUtils.hasText(userInfoResponse.getUnionid())) {
            accountAuthsService.save(_account.getId(), AccountAuths.IdentityType.WEIXIN_OPEN_UNIONID, userInfoResponse.getUnionid(), "", "");
        }

        /* 通过队列实现注册奖励积分 */
        stringRedisTemplate.opsForList().leftPush("yxl:integral-cash:integral_to_sign_up", _account.getId());

        return _account;
    }

}
