package com.studio.nx.help.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.help.controller.search.HelpGroupSearchDto;
import com.studio.nx.help.dto.*;
import com.studio.nx.help.entity.HelpContent;
import com.studio.nx.help.entity.HelpGroup;
import com.studio.nx.help.mapper.HelpGroupMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Slf4j
@Service
public class HelpGroupService {

    @Autowired
    private HelpGroupMapper mapper;

    @Autowired
    private HelpContentService helpContentService;

    /**
     * 功能描述：查询[问题分类]详情
     *
     * @param id 标识
     * @return
     */
    public HelpGroup findById(String id) {
        return mapper.selectById(id);
    }

    /**
     * 功能描述：查询[问题分类]列表
     */
    public BasePageResponse<HelpGroupListDto> page(HelpGroupSearchDto dto, Integer page, Integer size) {
        LambdaQueryWrapper<HelpGroup> lambdaQueryWrapper = new QueryWrapper().lambda();
        if (!StringUtils.isEmpty(dto.getTitle())) {
            lambdaQueryWrapper.like(HelpGroup::getName, dto.getTitle());
        }
        if (dto.getStatus() != null) {
            lambdaQueryWrapper.eq(HelpGroup::getIsShow, dto.getStatus());
        }

        // 列表只展示第一层分类
        lambdaQueryWrapper.eq(HelpGroup::getParentId, "-1");
        lambdaQueryWrapper.eq(HelpGroup::getIsDel, Boolean.FALSE);
        lambdaQueryWrapper.orderByDesc(HelpGroup::getSort);

        Page<HelpGroup> p = mapper.selectPage(new Page(page, size), lambdaQueryWrapper);

        // 查出列表其下级列表要连锁查询
        List<HelpGroupListDto> list = new ArrayList<HelpGroupListDto>();
        for (HelpGroup group : p.getRecords()) {
            HelpGroupListDto helpGroupListDto = new HelpGroupListDto();
            BeanUtils.copyProperties(group, helpGroupListDto);
            list.add(helpGroupListDto);
        }
        list = cycList(list);

        return new BasePageResponse().setTotal(p.getTotal()).setData(list);
    }

    /**
     * 循环查询子类分类
     *
     * @param list [description]
     * @return [description]
     */
    private List<HelpGroupListDto> cycList(List<HelpGroupListDto> list) {
        for (HelpGroupListDto item : list) {
            List<HelpGroupListDto> childrenDto = new ArrayList<HelpGroupListDto>();
            List<HelpGroup> children = mapper.selectList(new QueryWrapper<HelpGroup>().lambda()
                    .eq(HelpGroup::getParentId, item.getId())
                    .orderByDesc(HelpGroup::getSort)
            );

            for (HelpGroup helpGroup : children) {
                HelpGroupListDto helpGroupListDto = new HelpGroupListDto();
                BeanUtils.copyProperties(helpGroup, helpGroupListDto);
                childrenDto.add(helpGroupListDto);
            }
            item.setChildren(childrenDto);
        }

        return list;
    }

    /**
     * 功能描述：保存[问题分类]
     *
     * @param dto
     */
    public void saveItem(HelpGroupItemDto dto) {
        HelpGroup _helpGroup = new HelpGroup();
        _helpGroup.setName(dto.getName());
        _helpGroup.setSort(dto.getSort());
        _helpGroup.setIsShow(dto.getIsShow());
        _helpGroup.setParentId(dto.getParentId());
        mapper.insert(_helpGroup);
    }

    /**
     * 功能描述：更新[问题分类]
     *
     * @param id
     * @param dto
     */
    public void updateItem(String id, HelpGroupItemDto dto) {
        HelpGroup entity = findById(id);
        if (!Optional.ofNullable(entity).isPresent()) {
            return;
        }

        entity.setName(dto.getName());
        entity.setSort(dto.getSort());
        entity.setIsShow(dto.getIsShow());
        entity.setParentId(dto.getParentId());
        mapper.updateById(entity);
    }

    /**
     * 功能描述：替换[问题分类]显示状态
     *
     * @param id
     */
    public void handleIsShow(String id) {
        HelpGroup entity = findById(id);
        if (!Optional.ofNullable(entity).isPresent()) {
            return;
        }

        entity.setIsShow(!entity.getIsShow());
        mapper.updateById(entity);
    }

    /**
     * 功能描述：替换[问题分类]删除状态
     *
     * @param id
     */
    public void handleIsDel(String id) {
        HelpGroup entity = findById(id);
        if (!Optional.ofNullable(entity).isPresent()) {
            return;
        }

        entity.setIsDel(!entity.getIsDel());
        mapper.updateById(entity);
    }

    /**
     * 查询[帮助]分类和内容
     * 清除列表中某层数组为空的
     *
     * @return [description]
     */
    public List<HelpGroupTopListDto> getH5List() {
        // 首层查询和定义
        List<HelpGroup> helpGroupList1 = getListByH5("-1");
        List<HelpGroupTopListDto> result = new ArrayList<>();

        for (HelpGroup item : helpGroupList1) {
            // 首层单项复制
            HelpGroupTopListDto helpGroupTopListDto = new HelpGroupTopListDto();
            BeanUtils.copyProperties(item, helpGroupTopListDto);

            // 次层查询和定义
            List<HelpGroup> helpGroupList2 = getListByH5(item.getId());
            List<HelpGroupSecondListDto> helpGroupSecondListDtoList = new ArrayList<>();

            for (HelpGroup s : helpGroupList2) {
                // 次层单项复制
                HelpGroupSecondListDto helpGroupSecondListDto = new HelpGroupSecondListDto();
                BeanUtils.copyProperties(s, helpGroupSecondListDto);

                // 内容层查询和定义
                List<HelpContent> helpContentList = helpContentService.getListByH5(s.getId());
                List<HelpContentItemDto> contentItemDtoList = new ArrayList<>();

                for (HelpContent c : helpContentList) {
                    // 内容层单项复制
                    HelpContentItemDto contentItemDto = new HelpContentItemDto();
                    BeanUtils.copyProperties(c, contentItemDto);
                    contentItemDtoList.add(contentItemDto);
                }
                if (contentItemDtoList.size() > 0) {
                    helpGroupSecondListDto.setList(contentItemDtoList);
                    helpGroupSecondListDtoList.add(helpGroupSecondListDto);
                }
            }
            if (helpGroupSecondListDtoList.size() > 0) {
                helpGroupTopListDto.setChildren(helpGroupSecondListDtoList);
                result.add(helpGroupTopListDto);
            }
        }
        return result;
    }

    /***
     * 精确查寻H5的分类列表
     * @param parentId
     * @return
     */
    private List<HelpGroup> getListByH5(String parentId) {
        return mapper.selectList(new QueryWrapper<HelpGroup>().lambda()
                .eq(HelpGroup::getParentId, parentId)
                .eq(HelpGroup::getIsShow, true)
                .eq(HelpGroup::getIsDel, false)
        );
    }

}
