package com.studio.nx.oss.alioss.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.IdUtil;
import com.aliyun.oss.ClientConfiguration;
import com.aliyun.oss.HttpMethod;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.common.auth.DefaultCredentialProvider;
import com.aliyun.oss.model.*;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.http.MethodType;
import com.aliyuncs.http.ProtocolType;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.profile.IClientProfile;
import com.aliyuncs.sts.model.v20150401.AssumeRoleRequest;
import com.aliyuncs.sts.model.v20150401.AssumeRoleResponse;
import com.studio.nx.oss.alioss.config.AliOssYaml;
import com.studio.nx.oss.dto.FileResponse;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.net.URL;
import java.util.Date;

/**
 * 功能描述：
 * https://help.aliyun.com/document_detail/32008.html
 * todo https://help.aliyun.com/knowledge_detail/39533.html
 *
 * @Author 梁展鹏
 * @Date 2021/8/3 16:53
 */
@Slf4j
@Service
public class AliOssService {

    public static final int DURATION = 300;

    // 限速100KB/s，即819200bit/s。
    public static final int UPLOAD_LIMIT_SPEED = 512 * 1024 * 8;
    public static final int DOWNLOAD_LIMIT_SPEED = 512 * 1024 * 8;

    @Autowired
    private AliOssYaml aliOssYaml;

    public OSSClient ossClient() {
        return new OSSClient(aliOssYaml.getEndpoint(), new DefaultCredentialProvider(aliOssYaml.getAccessKey(), aliOssYaml.getSecretKey()), new ClientConfiguration());
    }

    @SneakyThrows
    public AssumeRoleResponse sts() {
        // STS
        DefaultProfile.addEndpoint("", "", "Sts", "sts.cn-hangzhou.aliyuncs.com");
        // add by 梁展鹏 20210812 这里的accessKeyId、secret均为梁展鹏子账号申请的秘钥
        IClientProfile profile = DefaultProfile.getProfile("cn-hangzhou", "LTAI5tMJbgaKNMjosPkyoEEH", "avcAVTGgHzRdWm76VFIfcmpsyss3nE");
        DefaultAcsClient client = new DefaultAcsClient(profile);

        final AssumeRoleRequest request = new AssumeRoleRequest();
        request.setMethod(MethodType.POST);
        request.setProtocol(ProtocolType.HTTPS);
        request.setDurationSeconds(60 * 60 * 1L); // 临时凭证的有效期，单位是s，最小为900，最大为3600；
        request.setRoleArn("acs:ram::1543622644542940:role/aliyunosstokengeneratorrole");  // 要扮演的角色ID
        request.setRoleSessionName("external-username");

        // 生成临时授权凭证
        final AssumeRoleResponse response = client.getAcsResponse(request);

        String appKey = response.getCredentials().getAccessKeyId();  // 临时凭据AccessKeyId
        String appSecret = response.getCredentials().getAccessKeySecret();  // 临时凭据AccessKeySecret
        String securityToken = response.getCredentials().getSecurityToken();
        String expiration = response.getCredentials().getExpiration();
        return response;
    }

    /**
     * 功能描述：存储空间 - 创建存储空间
     * https://help.aliyun.com/document_detail/32012.html
     *
     * @param bucketName 分桶名称
     */
    @SneakyThrows
    public void createBucket(String bucketName) {
        StopWatch watch = new StopWatch();
        watch.start();

        OSSClient ossClient = ossClient();

        if (!ossClient.doesBucketExist(bucketName)) {
            // 创建CreateBucketRequest对象。
            CreateBucketRequest createBucketRequest = new CreateBucketRequest(bucketName);

            // 如果创建存储空间的同时需要指定存储类型和数据容灾类型, 请参考如下代码。
            // 此处以设置存储空间的存储类型为标准存储为例介绍。
            createBucketRequest.setStorageClass(StorageClass.Standard);

            // 数据容灾类型默认为本地冗余存储，即DataRedundancyType.LRS。如果需要设置数据容灾类型为同城冗余存储，请设置为DataRedundancyType.ZRS。
            createBucketRequest.setDataRedundancyType(DataRedundancyType.LRS);

            // 设置存储空间的权限为公共读，默认为私有。
            createBucketRequest.setCannedACL(CannedAccessControlList.Private);

            // 创建存储空间。
            Bucket bucket = ossClient.createBucket(createBucketRequest);
            Assert.isTrue(bucketName.equals(bucket.getName()), "创建Bucket失败");
        }

        ossClient.shutdown();

        watch.stop();
        log.info("createBucket [{}] ms", watch.getTotalTimeMillis());
    }

    /**
     * 功能描述：存储空间 - 删除存储空间
     * https://help.aliyun.com/document_detail/145213.html
     *
     * @param bucketName
     */
    public void deleteBucket(String bucketName) {
        StopWatch watch = new StopWatch();
        watch.start();

        OSSClient ossClient = ossClient();
        ossClient.deleteBucket(bucketName);
        ossClient.shutdown();

        watch.stop();
        log.info("deleteBucket [{}] ms", watch.getTotalTimeMillis());
    }

    /**
     * 功能描述：上传文件 - 简单上传
     * https://help.aliyun.com/document_detail/84781.html
     *
     * @param bucketName
     * @param prefix
     * @param file
     * @param metadata
     * @return
     */
    @SneakyThrows
    public FileResponse putObject(String bucketName, String prefix, MultipartFile file, ObjectMetadata metadata) {
        StopWatch watch = new StopWatch();
        watch.start();

//        createBucket(bucketName);

        /* 上传后的文件名称 */
        String originalFilename = file.getOriginalFilename();
        String extension = FileUtil.extName(originalFilename);
        String filename = String.format("%s_%s.%s", DateUtil.format(new Date(), "yyyyMMddHHmm"), IdUtil.simpleUUID(), extension);

        /* 处理目录前缀 */
        String filePath = filename;
        if (StringUtils.hasText(prefix)) {
            filePath = prefix.trim() + File.separator + filename;
        }
        if (filePath.startsWith("/")) {
            filePath = filePath.substring(1);
        }

        OSSClient ossClient = ossClient();

        InputStream is = file.getInputStream();
        PutObjectRequest putObjectRequest = new PutObjectRequest(bucketName, filePath, is, metadata);
        putObjectRequest.setMetadata(metadata);
//        putObjectRequest.setTrafficLimit(UPLOAD_LIMIT_SPEED);
        PutObjectResult result = ossClient.putObject(putObjectRequest); //上传文件
        is.close();

        ossClient.shutdown();

        String fullPath = generatePresignedUrl(bucketName, filePath, DURATION, DOWNLOAD_LIMIT_SPEED);
        FileResponse _response = new FileResponse();
        _response.setOriginalFilename(originalFilename); // 原始文件名称
        _response.setFilename(filename); // 上传后的文件名称
        _response.setExtension(extension); // 文件扩展名称
        _response.setSize(file.getSize()); // 文件大小
        _response.setFullPath(fullPath); // 访问全路径
        _response.setRelativePath(new URI(fullPath).getPath().substring(1)); // 访问相对路径

        watch.stop();
        log.info("putObject [{}] ms", watch.getTotalTimeMillis());

        return _response;
    }

    /**
     * 功能描述：上传文件 - 简单上传
     * https://help.aliyun.com/document_detail/84781.html
     *
     * @param bucketName
     * @param prefix
     * @param localPath
     * @param metadata
     * @return
     */
    @SneakyThrows
    public FileResponse putObject(String bucketName, String prefix, String localPath, ObjectMetadata metadata) {
        StopWatch watch = new StopWatch();
        watch.start();

//        createBucket(bucketName);

        /* 上传后的文件名称 */
        String mainName = FileNameUtil.mainName(localPath);
        String extName = FileNameUtil.extName(localPath);
        String filename = FileNameUtil.getName(localPath);
        File localFile = FileUtil.file(localPath);

        /* 处理目录前缀 */
        String filePath = filename;
        if (StringUtils.hasText(prefix)) {
            filePath = prefix.trim() + File.separator + filename;
        }
        if (filePath.startsWith("/")) {
            filePath = filePath.substring(1);
        }

        PutObjectRequest putObjectRequest = new PutObjectRequest(bucketName, filePath, localFile);
        putObjectRequest.setMetadata(metadata);
//        putObjectRequest.setTrafficLimit(UPLOAD_LIMIT_SPEED);

        OSSClient ossClient = ossClient();
        PutObjectResult result = ossClient.putObject(putObjectRequest); //上传文件
        ossClient.shutdown();

        String fullPath = generatePresignedUrl(bucketName, filePath, DURATION, DOWNLOAD_LIMIT_SPEED);
        FileResponse _response = new FileResponse();
        _response.setOriginalFilename(mainName); // 原始文件名称
        _response.setFilename(filename); // 上传后的文件名称
        _response.setExtension(extName); // 文件扩展名称
        _response.setSize(localFile.length()); // 文件大小
        _response.setFullPath(fullPath); // 访问全路径
        _response.setRelativePath(new URI(fullPath).getPath().substring(1)); // 访问相对路径

        watch.stop();
        log.info("putObject [{}] ms", watch.getTotalTimeMillis());

        return _response;
    }

    /**
     * 功能描述：管理文件 - 拷贝文件
     * 允许指定源存储空间和源文件，以及目标存储空间和目标文件。拷贝后，目标文件的内容及元信息与源文件相同，称为简单拷贝。
     * https://help.aliyun.com/document_detail/84843.html
     *
     * @param sourceBucketName
     * @param sourceFilePath
     * @param destinationBucketName
     * @param targetFilePath
     */
    public void copyObject(String sourceBucketName, String sourceFilePath, String destinationBucketName, String targetFilePath) {
        StopWatch watch = new StopWatch();
        watch.start();

//        createBucket(destinationBucketName);

        OSSClient ossClient = ossClient();
        if (sourceFilePath.startsWith("/")) {
            sourceFilePath = sourceFilePath.substring(1);
        }
        if (targetFilePath.startsWith("/")) {
            targetFilePath = targetFilePath.substring(1);
        }
        CopyObjectResult result = ossClient.copyObject(sourceBucketName, sourceFilePath, destinationBucketName, targetFilePath);
        ossClient.shutdown();

        // 读取文件内容。
        watch.stop();
        log.info("copyObject [{}] ms", watch.getTotalTimeMillis());
    }

    /**
     * 功能描述：文件下载 - 流式下载
     * https://help.aliyun.com/document_detail/84823.html
     *
     * @param fileName
     */
    @SneakyThrows
    public void download(String bucketName, String fileName, HttpServletResponse response) {
        StopWatch watch = new StopWatch();
        watch.start();

        if (fileName.startsWith("/")) {
            fileName = fileName.substring(1);
        }

        OSSClient ossClient = ossClient();
        OSSObject ossObject = ossClient.getObject(bucketName, fileName);

        response.setHeader("Content-Disposition", "attachment;filename=" + fileName.substring(fileName.lastIndexOf("/") + 1, fileName.length()));
        response.setContentType("application/x-download");
        response.setCharacterEncoding("UTF-8");
        IOUtils.copy(ossObject.getObjectContent(), response.getOutputStream());

        watch.stop();
        log.info("download [{}] ms", watch.getTotalTimeMillis());
    }

    /**
     * 功能描述：文件下载 - 下载到本地文件
     * https://help.aliyun.com/document_detail/84824.html
     *
     * @param fileName
     * @return
     */
    @SneakyThrows
    public void download(String bucketName, String fileName, String downloadLocalPath) {
        StopWatch watch = new StopWatch();
        watch.start();

        /* 创建下载目录 */
        File file = new File(downloadLocalPath);
        FileUtil.mkdir(file.getParent());

        if (fileName.startsWith("/")) {
            fileName = fileName.substring(1);
        }

        OSSClient ossClient = ossClient();
        ossClient.getObject(new GetObjectRequest(bucketName, fileName), new File(downloadLocalPath));
        ossClient.shutdown();

        watch.stop();
        log.info("download [{}] ms", watch.getTotalTimeMillis());
    }

    /**
     * 功能描述：照片处理 - 流式下载
     * https://help.aliyun.com/document_detail/47505.html
     * https://gosspublic.alicdn.com/image/index.html
     *
     * @param fileName
     */
    @SneakyThrows
    public void processAndDownload(String bucketName, String fileName, String downloadLocalPath) {
        StopWatch watch = new StopWatch();
        watch.start();

        /* 创建下载目录 */
        File file = new File(downloadLocalPath);
        FileUtil.mkdir(file.getParent());

        if (fileName.startsWith("/")) {
            fileName = fileName.substring(1);
        }

        OSSClient ossClient = ossClient();
        GetObjectRequest request = new GetObjectRequest(bucketName, fileName);
        request.setProcess("image/auto-orient,1/format,jpg"); // 参数参考方法注释地址
        ossClient.getObject(request, new File(downloadLocalPath));
        ossClient.shutdown();

        watch.stop();
        log.info("download [{}] ms", watch.getTotalTimeMillis());
    }

    /**
     * 功能描述：获取下载链接
     * https://help.aliyun.com/document_detail/129781.html
     *
     * @param bucketName
     * @param duration     有效期（秒）
     * @param trafficLimit 限速
     * @return
     */
    @SneakyThrows
    public String generatePresignedUrl(String bucketName, String fileName, int duration, int trafficLimit) {
        StopWatch watch = new StopWatch();
        watch.start();

        if (fileName.startsWith("/")) {
            fileName = fileName.substring(1);
        }

        // 创建限速下载的url, 有效期60s。
        GeneratePresignedUrlRequest request = new GeneratePresignedUrlRequest(bucketName, fileName, HttpMethod.GET);
        Date date = new Date();
        date.setTime(date.getTime() + duration * 1000);
        request.setExpiration(date);
        if (trafficLimit > 0) {
            request.setTrafficLimit(trafficLimit);
        }

        OSSClient ossClient = ossClient();
        URL signedUrl = ossClient.generatePresignedUrl(request);
        ossClient.shutdown();

        // 读取文件内容。
        watch.stop();
        log.info("generatePresignedUrl [{}] ms", watch.getTotalTimeMillis());

        return signedUrl.toString();
    }

    /**
     * 功能描述：
     *
     * @param bucketName bucket名称
     * @param filePath   文件路径
     */
    public void deleteFile(String bucketName, String filePath) {
        StopWatch watch = new StopWatch();
        watch.start();

        if (filePath.startsWith("/")) {
            filePath = filePath.substring(1);
        }

        OSSClient ossClient = ossClient();
        ossClient.deleteObject(bucketName, filePath);
        ossClient.shutdown();

        watch.stop();
        log.info("deleteFile [{}] ms", watch.getTotalTimeMillis());
    }

    /**
     * 通过文件名判断并获取OSS服务文件上传时文件的contentType
     *
     * @param fileName 文件名
     * @return 文件的contentType
     */
    public String getContentType(String fileName) {
        String fileExtension = fileName.substring(fileName.lastIndexOf(".") + 1);

        if ("bmp".equalsIgnoreCase(fileExtension)) return "image/bmp";
        if ("gif".equalsIgnoreCase(fileExtension)) return "image/gif";
        if ("jpeg".equalsIgnoreCase(fileExtension) || "jpg".equalsIgnoreCase(fileExtension)) return "image/jpeg";
        if ("png".equalsIgnoreCase(fileExtension)) return "image/png";
        if ("html".equalsIgnoreCase(fileExtension)) return "text/html";
        if ("txt".equalsIgnoreCase(fileExtension)) return "text/plain";
        if ("vsd".equalsIgnoreCase(fileExtension)) return "application/vnd.visio";
        if ("ppt".equalsIgnoreCase(fileExtension) || "pptx".equalsIgnoreCase(fileExtension))
            return "application/vnd.ms-powerpoint";
        if ("doc".equalsIgnoreCase(fileExtension) || "docx".equalsIgnoreCase(fileExtension))
            return "application/msword";
        if ("xml".equalsIgnoreCase(fileExtension)) return "text/xml";

        return "text/html";
    }

}
