package com.studio.nx.oss.local.service;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import com.studio.nx.oss.dto.FileResponse;
import com.studio.nx.oss.local.config.LocalYaml;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.Date;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/6/12 08:45
 */
@Slf4j
@Service
public class LocalService {

    @Value("${server.servlet.context-path}")
    private String serverServletContextPath;

    @Value("${server.port}")
    private String serverPort;

    @Autowired
    private LocalYaml localYaml;

    private String getLocalUrl() {
        return String.format("%s%s", localYaml.getHost(), localYaml.getMvcPath());
    }

    /**
     * 功能描述：截取相对路径
     * todo 与MinioService相似
     *
     * @param fullPath 全路径
     * @return
     */
    public String cutPath(String fullPath) {
        /* 同时不满足http、https条件的直接返回 */
        if (!fullPath.startsWith("http://") && !fullPath.startsWith("https://")) {
            return fullPath;
        }

        /* 兼容没有请求参数的情况 */
        String prefixUrl = fullPath;
        if (fullPath.indexOf("?") != -1) {
            prefixUrl = fullPath.substring(0, fullPath.indexOf("?")); // 剔除请求参数
        }

        if (prefixUrl.startsWith("http://")) {
            prefixUrl = prefixUrl.replace("http://", "https://");
        }
        String localUrl = getLocalUrl();
        if (localUrl.startsWith("http://")) {
            localUrl = localUrl.replace("http://", "https://");
        }

        return prefixUrl.substring(localUrl.length());
    }

    /**
     * 功能描述：获取全路径
     *
     * @param path
     * @return
     */
    public String fullPath(String path) {
        return getLocalUrl() + path;
    }

    /**
     * 功能描述：判断本地文件是否存在
     * @param path
     * @return
     */
    public Boolean isExists(String path) {
        return FileUtil.exist(localYaml.getPath() + path);
    }

    /**
     * 功能描述：上传文件
     *
     * @param file   文件
     * @param prefix
     * @return
     */
    public FileResponse upload(MultipartFile file, String prefix) {
        StopWatch watch = new StopWatch();
        watch.start();

        String filePath = localYaml.getPath();
        if (file.isEmpty() || !StringUtils.hasText(filePath)) {
            log.error("[nx-oss:local] [文件]或[上传目录]为空");
            return null;
        }

        String originalFileName = file.getOriginalFilename(); // 源文件名(包含后缀)
        String extension = FileUtil.extName(originalFileName); // 源文件后缀名
        String newFileName = String.format("%s.%s", IdUtil.simpleUUID(), extension);

        /* 文件存储到本地的相对路径（/${prefix}/fileName.ext） */
        StringBuilder tempPathSB = new StringBuilder();
        /* 处理目录前缀 */
        if (StringUtils.hasText(prefix)) {
            prefix = prefix.trim().startsWith(File.separator) ? prefix.trim() : File.separator + prefix.trim();
        } else {
            prefix = File.separator + "tmp";
        }
        tempPathSB.append(prefix);
        tempPathSB.append(File.separator);
        tempPathSB.append(DateUtil.format(new Date(), "yyyyMM"));
        tempPathSB.append(File.separator);
        tempPathSB.append(newFileName);

        /* 文件存储到本地的绝对路径（/${filePath}/${prefix}/fileName.ext） */
        StringBuilder localTempPathSB = new StringBuilder().append(filePath).append(tempPathSB.toString());
        log.debug("[nx-oss:local] 文件存储目录：[{}]", localTempPathSB.toString());

        /* 文件存储到本地目录 */
        File destFile = new File(localTempPathSB.toString());
        if (!destFile.getParentFile().exists()) {
            destFile.getParentFile().mkdirs();
        }
        log.debug("[nx-oss:local] 文件存储到本地目录：[{}]", destFile.getAbsolutePath());

        try {
            /* 保存文件 */
            file.transferTo(destFile.getAbsoluteFile());

            return new FileResponse()
                    .setOriginalFilename(originalFileName) // 原始文件名称
                    .setFilename(newFileName) // 上传后的文件名称
                    .setExtension(extension) // 文件扩展名称
                    .setSize(file.getSize()) // 文件大小
                    .setFullPath(String.format("%s%s%s", localYaml.getHost(), localYaml.getMvcPath(), tempPathSB.toString())) // 访问全路径
                    .setRelativePath(String.format("%s%s", localYaml.getMvcPath(), tempPathSB)) // 访问相对路径
                    .setLocalTempPath(localTempPathSB.toString()); // 本地存储路径
        } catch (Exception e) {
            log.error("[nx-oss:local] 上传文件异常：[{}]", e);
            return null;
        } finally {
            watch.stop();
            log.debug("processing in [{}] ms", watch.getTotalTimeMillis());
        }
    }

}
