package com.studio.nx.oss.minio.service;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.file.FileNameUtil;
import cn.hutool.core.util.IdUtil;
import com.google.common.collect.Iterables;
import com.studio.nx.oss.dto.FileResponse;
import com.studio.nx.oss.minio.config.MinioYaml;
import io.minio.*;
import io.minio.http.Method;
import io.minio.messages.Item;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.net.URI;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 功能描述：
 * http://docs.minio.org.cn/docs/master/java-client-quickstart-guide
 * https://blog.csdn.net/weixin_42170236/article/details/109356921
 * https://blog.csdn.net/wangdingfeng5141/article/details/109642706
 *
 * @Author 梁展鹏
 * @Date 2021/6/9 15:05
 */
@Slf4j
@Service
public class MinioService {

    @Autowired
    private MinioYaml minioYaml;

    @Autowired
    private MinioClient minioClient;

    @Autowired
    private MinioClient localMinioClient;

    private String getMinioUrl() {
        return String.format("%s:%s", minioYaml.getEndpoint(), minioYaml.getPort());
    }

    /**
     * 功能描述：截取相对路径（包含bucketName）
     * todo 与LocalService相似
     * ex:
     * 1. fullPath：http://430jy.uutime.cn:19000/kindergarten-apply/1401825572046675970/e046162efb4047b8b24d82ee55be0452.png?Content-Disposition=attachment%3B%20filename%3D%221401825572046675970%2Fe046162efb4047b8b24d82ee55be0452.png%22&X-Amz-Algorithm=AWS4-HMAC-SHA256&X-Amz-Credential=admin%2F20210625%2F%2Fs3%2Faws4_request&X-Amz-Date=20210625T014103Z&X-Amz-Expires=432000&X-Amz-SignedHeaders=host&X-Amz-Signature=ca2b894a29e9c59a502ac95535cae77e125a5efff76d7de528633d4f854dd2a2
     * 2. fullPath：http://430jy.uutime.cn:19000/kindergarten-apply/1401825572046675970/e046162efb4047b8b24d82ee55be0452.png
     * 3. return：/kindergarten-apply/1401825572046675970/e046162efb4047b8b24d82ee55be0452.png
     *
     * @param fullPath 全路径
     * @return
     */
    @SneakyThrows
    public String cutPath(String fullPath) {
        return new URI(fullPath).getPath();
    }

    /**
     * 功能描述：截取BucketName
     * ex:
     * 1. relativePath：/kindergarten-apply/1401825572046675970/e046162efb4047b8b24d82ee55be0452.png
     * 2. return：kindergarten-apply
     *
     * @param relativePath 数据库保存的Minio路径（包含BucketName）
     * @return
     */
    public String cutBucketName(String relativePath) {
        String minioPath = relativePath;
        /* 统一数据格式，都没有包含/ */
        if (relativePath.startsWith("/")) {
            minioPath = minioPath.substring(1, relativePath.length());
        }

        /* 截取第一段作为BucketName */
        return minioPath.split("/")[0];
    }

    /**
     * 功能描述：截取Minio的filename路径（不包含BucketName），不带根路径
     * ex:
     * 1. relativePath：/kindergarten-apply/1401825572046675970/e046162efb4047b8b24d82ee55be0452.png
     * 2. bucketName：kindergarten-apply
     * 2. return：1401825572046675970/e046162efb4047b8b24d82ee55be0452.png
     *
     * @param bucketName   Minio路径第一段即为BucketName
     * @param relativePath 数据库保存的Minio路径（包含BucketName）
     * @return
     */
    public String cutFilename(String bucketName, String relativePath) {
        return relativePath.substring(String.format("/%s", bucketName).length() + 1, relativePath.length());
    }

    /**
     * 功能描述：检验分桶是否存在，不存在则新建
     *
     * @param bucketName 分桶名称
     */
    // todo lombok的注解学习
    @SneakyThrows
    public void makeBucket(String bucketName) {
        // todo Args是什么设计模式???
        if (!localMinioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build())) {
            localMinioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).build());
        }
    }

    /**
     * 功能描述：判断Minio文件是否存在
     * @param bucketName
     * @param filename
     * @return
     */
    public Boolean isExists(String bucketName, String filename) {
        try {
            localMinioClient.statObject(StatObjectArgs.builder()
                    .bucket(bucketName)
                    .object(filename)
                    .build()
            );
            return Boolean.TRUE;
        } catch (Exception e) {
            log.error("文件不存在 bucketName: [{}] filename: [{}] [{}]", bucketName, filename, e);
            return Boolean.FALSE;
        }
    }

    /**
     * 功能描述：File方式上传图片
     *
     * @param bucketName   分桶名称
     * @param prefix       待上传文件的前缀路径
     * @param originalPath 待上传文件的全路径
     * @param headers
     * @param userMetadata
     */
    @SneakyThrows
    public FileResponse uploadObject(String bucketName, String prefix, String originalPath, Map<String, String> headers, Map<String, String> userMetadata) {
        StopWatch watch = new StopWatch();
        watch.start();

        /* 检验分桶是否存在，不存在则新建 */
        makeBucket(bucketName);

        /* 上传后的文件名称 */
        String mainName = FileNameUtil.mainName(originalPath);
        String extName = FileNameUtil.extName(originalPath);
        File file = FileUtil.file(originalPath);
        String originalFilename = String.format("%s.%s", mainName, extName);

        /* 处理目录前缀 */
        String minioFilename = originalFilename;
        if (StringUtils.hasText(prefix)) {
            minioFilename = prefix.trim() + File.separator + originalFilename;
        }

        /* 上传 */
        localMinioClient.uploadObject(
                UploadObjectArgs.builder()
                        .bucket(bucketName)
                        .filename(originalPath)
                        .object(minioFilename)
                        .headers(headers)
                        .userMetadata(userMetadata)
                        .build()
        );

        String fullPath = getPresignedObjectUrl(bucketName, minioFilename, 30, TimeUnit.SECONDS);
        FileResponse _response = new FileResponse();
        _response.setOriginalFilename(originalFilename); // 原始文件名称
        _response.setFilename(originalFilename); // 上传后的文件名称
        _response.setExtension(extName); // 文件扩展名称
        _response.setSize(file.length()); // 文件大小
        _response.setFullPath(fullPath); // 访问全路径
        _response.setRelativePath(cutPath(fullPath)); // 访问相对路径

        watch.stop();
        log.info("uploadObject in [{}] ms", watch.getTotalTimeMillis());

        return _response;
    }

    /**
     * 功能描述：输入流方式上传图片
     *
     * @param bucketName   分桶名称
     * @param file         输入流
     * @param headers
     * @param userMetadata
     */
    @SneakyThrows
    public FileResponse putObject(String bucketName, String prefix, MultipartFile file, Map<String, String> headers, Map<String, String> userMetadata) {
        StopWatch watch = new StopWatch();
        watch.start();

        /* 检验分桶是否存在，不存在则新建 */
        makeBucket(bucketName);

        /* 上传后的文件名称 */
        String originalFilename = file.getOriginalFilename();
        String extension = FileUtil.extName(originalFilename);
        String filename = String.format("%s.%s", IdUtil.simpleUUID(), extension);

        /* 处理目录前缀 */
        String minioFilename = filename;
        if (StringUtils.hasText(prefix)) {
            minioFilename = prefix.trim() + File.separator + filename;
        }

        /* 上传 */
        localMinioClient.putObject(
                PutObjectArgs.builder()
                        .bucket(bucketName)
                        .stream(file.getInputStream(), file.getInputStream().available(), -1)
                        .object(minioFilename)
                        .headers(headers)
                        .userMetadata(userMetadata)
                        .build()
        );

        String fullPath = getPresignedObjectUrl(bucketName, minioFilename, 30, TimeUnit.SECONDS);
        FileResponse _response = new FileResponse();
        _response.setOriginalFilename(file.getOriginalFilename()); // 原始文件名称
        _response.setFilename(filename); // 上传后的文件名称
        _response.setExtension(extension); // 文件扩展名称
        _response.setSize(file.getSize()); // 文件大小
        _response.setFullPath(fullPath); // 访问全路径
        _response.setRelativePath(cutPath(fullPath)); // 访问相对路径

        watch.stop();
        log.info("putObject in [{}] ms", watch.getTotalTimeMillis());

        return _response;
    }

    /**
     * 功能描述：拷贝文件(只对业务操作)
     *
     * @param bucketName
     * @param filename
     * @param targetBucketName
     * @param targetFilename
     * @return
     */
    @SneakyThrows
    public FileResponse copyObject(String bucketName, String filename, String targetBucketName, String targetFilename) {
        StopWatch watch = new StopWatch();
        watch.start();

        /* 检验分桶是否存在，不存在则新建 */
        makeBucket(targetBucketName);

        localMinioClient.copyObject(CopyObjectArgs.builder()
                .source(CopySource.builder()
                        .bucket(bucketName)
                        .object(filename)
                        .build()
                )
                .bucket(targetBucketName)
                .object(targetFilename)
                .build()
        );

        String fullPath = getPresignedObjectUrl(targetBucketName, targetFilename, 30, TimeUnit.SECONDS);
        FileResponse _response = new FileResponse();
        _response.setOriginalFilename(""); // 原始文件名称
        _response.setFilename(filename); // 上传后的文件名称
        _response.setExtension(""); // 文件扩展名称
        _response.setSize(null); // 文件大小
        _response.setFullPath(fullPath); // 访问全路径
        _response.setRelativePath(cutPath(fullPath)); // 访问相对路径

        watch.stop();
        log.info("copyObject in [{}] ms", watch.getTotalTimeMillis());

        return _response;
    }

    /**
     * 功能描述：获取文件链接地址（带过期时间）
     *
     * @param bucketName 分桶名称
     * @param filename   上传后的文件名称
     * @return
     */
    @SneakyThrows
    public String getPresignedObjectUrl(String bucketName, String filename, int duration, TimeUnit unit) {
        StopWatch watch = new StopWatch();
        watch.start();

        String objectUrl = minioClient.getPresignedObjectUrl(
                GetPresignedObjectUrlArgs.builder()
                        .bucket(bucketName)
                        .object(filename)
                        .method(Method.GET)
                        .expiry(duration, unit)
                        .build());

        watch.stop();
        log.info("getPresignedObjectUrl in [{}] ms", watch.getTotalTimeMillis());

        return objectUrl;
    }

    /**
     * 功能描述：下载Minio文件到本地目录
     *
     * @param bucketName   分桶名称
     * @param filename     上传后的文件名称
     * @param downloadPath 本地目录全路径（包含文件名称.扩展名）
     */
    @SneakyThrows
    public void downloadLocal(String bucketName, String filename, String downloadPath) {
        StopWatch watch = new StopWatch();
        watch.start();

        /* 创建下载目录 */
        File file = new File(downloadPath);
        FileUtil.mkdir(file.getParent());

        localMinioClient.downloadObject(
                DownloadObjectArgs.builder()
                        .bucket(bucketName)
                        .object(filename)
                        .filename(downloadPath)
                        .build()
        );

        watch.stop();
        log.info("downloadLocal in [{}] ms", watch.getTotalTimeMillis());
    }

    /**
     * 功能描述：下载Minio文件到输出流
     *
     * @param bucketName 分桶名称
     * @param filename   上传后的文件名称
     * @return
     */
    @SneakyThrows
    public GetObjectResponse download(String bucketName, String filename) {
        StopWatch watch = new StopWatch();
        watch.start();

        GetObjectResponse response = localMinioClient.getObject(
                GetObjectArgs.builder()
                        .bucket(bucketName)
                        .object(filename)
                        .build()
        );

        watch.stop();
        log.info("download in [{}] ms", watch.getTotalTimeMillis());

        return response;
    }

    /**
     * 功能描述：删除Minio文件
     *
     * @param bucketName 分桶名称
     * @param filename   上传后的文件名称
     * @return
     */
    @SneakyThrows
    public void removeObject(String bucketName, String filename) {
        localMinioClient.removeObject(
                RemoveObjectArgs.builder()
                        .bucket(bucketName)
                        .object(filename)
                        .build()
        );
    }

    /**
     * 功能描述：删除Temp分桶的内容
     */
    @SneakyThrows
    public void removeBucket() {
        if (!localMinioClient.bucketExists(BucketExistsArgs.builder().bucket(minioYaml.getBucketName()).build())) {
            return;
        }

        StopWatch watch = new StopWatch();
        watch.start();

        Iterable<Result<Item>> list = localMinioClient.listObjects(ListObjectsArgs.builder().bucket(minioYaml.getBucketName()).build());
        log.info("桶：[{}], 可删除：[{}]个文件", minioYaml.getBucketName(), Iterables.size(list));
        for (Result<Item> itemResult : list) {
            Item item = itemResult.get();
            removeObject(minioYaml.getBucketName(), item.objectName());
        }

        watch.stop();
        log.info("桶：[{}], 共删除：[{}]个文件，共耗时：[{}] ms", watch.getTotalTimeMillis());
    }

}
