package com.studio.nx.oss.util;

import com.drew.imaging.ImageMetadataReader;
import com.drew.imaging.ImageProcessingException;
import com.drew.metadata.Directory;
import com.drew.metadata.Metadata;
import com.drew.metadata.Tag;
import com.drew.metadata.exif.ExifIFD0Directory;
import com.drew.metadata.exif.ExifSubIFDDirectory;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.io.IOException;

/**
 * 功能描述：
 *
 * @Author 梁展鹏
 * @Date 2021/5/28 23:45
 */
@Slf4j
public class MetadataUtil {

    public static void main(String[] args) {
        System.out.println("开始读取元数据：");
        ImageMetadata imageMetadata = MetadataUtil.parseImgInfo("/Users/liangzp/Downloads/a3abadd5a37e41df86d91992aa70af03.JPG");
        imageMetadata = MetadataUtil.parseImgInfo("/Users/liangzp/Downloads/a3abadd5a37e41df86d91992aa70af03_80_80_hd_logo.JPG");
        System.out.println("读取元数据完成。");
    }

    /**
     * 功能描述：图片信息获取metadata元数据信息
     *
     * @param originImgPath 需要解析的文件
     * @return
     */
    public static ImageMetadata parseImgInfo(String originImgPath) {
        File file = new File(originImgPath);
        ImageMetadata _imageMetadata = null;
        try {
            Metadata metadata = ImageMetadataReader.readMetadata(file);
            _imageMetadata = printImageTags(file, metadata);
        } catch (ImageProcessingException e) {
            log.error("error {}", e);
        } catch (IOException e) {
            log.error("error {}", e);
        }
        return _imageMetadata;
    }

    /**
     * 功能描述：读取metadata里面的信息
     *
     * @param originImgPath 源文件
     * @param metadata      metadata元数据信息
     * @return
     */
    private static ImageMetadata printImageTags(File originImgPath, Metadata metadata) {
        ImageMetadata _imageMetadata = new ImageMetadata();

        _imageMetadata.setName(originImgPath.getName());
        _imageMetadata.setSize(originImgPath.length());

        for (Directory directory : metadata.getDirectories()) {
            if ("ExifIFD0Directory".equalsIgnoreCase(directory.getClass().getSimpleName())) {
                _imageMetadata.setMake(directory.getString(ExifIFD0Directory.TAG_MAKE));
                _imageMetadata.setModel(directory.getString(ExifIFD0Directory.TAG_MAKE));
                _imageMetadata.setXResolution(directory.getString(ExifIFD0Directory.TAG_X_RESOLUTION));
                _imageMetadata.setYResolution(directory.getString(ExifIFD0Directory.TAG_Y_RESOLUTION));
            }
            if ("ExifSubIFDDirectory".equalsIgnoreCase(directory.getClass().getSimpleName())) {
                // 光圈F值=镜头的焦距/镜头光圈的直径
                _imageMetadata.setFnumber(directory.getString(ExifSubIFDDirectory.TAG_FNUMBER));
                _imageMetadata.setExposureTime(directory.getString(ExifSubIFDDirectory.TAG_EXPOSURE_TIME) + "秒");
                _imageMetadata.setIsoEquivalent(directory.getString(ExifSubIFDDirectory.TAG_ISO_EQUIVALENT));
                _imageMetadata.setFocalLength(directory.getString(ExifSubIFDDirectory.TAG_FOCAL_LENGTH) + "毫米");
                _imageMetadata.setDatetimeOriginal(directory.getString(ExifSubIFDDirectory.TAG_DATETIME_ORIGINAL));
                _imageMetadata.setExifImageWidth(directory.getString(ExifSubIFDDirectory.TAG_EXIF_IMAGE_WIDTH));
                _imageMetadata.setExifImagHeight(directory.getString(ExifSubIFDDirectory.TAG_EXIF_IMAGE_HEIGHT));
            }

            for (Tag tag : directory.getTags()) {
                String tagName = tag.getTagName();
                String desc = tag.getDescription();
                if (tagName.equals("Image Height")) { // 图片高度
                    _imageMetadata.setImgHeight(desc);
                } else if (tagName.equals("Image Width")) { // 图片宽度
                    _imageMetadata.setImgWidth(desc);
                } else if (tagName.equals("Date/Time Original")) { // 拍摄时间
                    _imageMetadata.setDateTime(desc);
                } else if (tagName.equals("GPS Altitude")) { // 海拔
                    _imageMetadata.setAltitude(desc);
                } else if (tagName.equals("GPS Latitude")) { // 纬度
                    _imageMetadata.setLatitude(pointToLatlong(desc));
                } else if (tagName.equals("GPS Longitude")) { // 经度
                    _imageMetadata.setLongitude(pointToLatlong(desc));
                }
            }
            for (String error : directory.getErrors()) {
                System.err.println("ERROR: " + error);
            }
        }
        return _imageMetadata;
    }

    /**
     * 经纬度格式 转换为 度分秒格式 ,如果需要的话可以调用该方法进行转换
     *
     * @param point 坐标点
     * @return
     */
    public static String pointToLatlong(String point) {
        Double du = Double.parseDouble(point.substring(0, point.indexOf("°")).trim());
        Double fen = Double.parseDouble(point.substring(point.indexOf("°") + 1, point.indexOf("'")).trim());
        Double miao = Double.parseDouble(point.substring(point.indexOf("'") + 1, point.indexOf("\"")).trim());
        Double duStr = du + fen / 60 + miao / 60 / 60;
        return duStr.toString();
    }

    @Data
    @NoArgsConstructor
    @AllArgsConstructor
    @Accessors(chain = true)
    public static class ImageMetadata {
        private String name; // 文件名称
        private Long size; // 文件大小
        private String make; // 相机制造商
        private String model; // 相机型号
        private String xResolution; // 水平分辨率
        private String yResolution; // 垂直分辨率

        private String fnumber; // 光圈值 f/
        private String exposureTime; // 曝光时间
        private String isoEquivalent; // ISO速度
        private String focalLength; // 焦距
        private String datetimeOriginal; // 照片时间
        private String exifImageWidth; // 宽
        private String exifImagHeight; // 高

        private String imgHeight; // 图片高度
        private String imgWidth; // 图片宽度
        private String dateTime; // 拍摄时间
        private String altitude; // 海拔
        private String latitude; // 纬度
        private String longitude; // 经度

    }
}
