package com.studio.nx.pay.weixin.service;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.studio.nx.pay.weixin.config.WeixinPayYaml;
import com.studio.nx.pay.weixin.dto.PaySuccessNotifyResponse;
import com.studio.nx.pay.weixin.dto.WeixinPrePayResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigDecimal;

/**
 * 功能描述：微信支付 - APP支付
 * https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_2_1.shtml
 *
 * @Author 梁展鹏
 * @Date 2021/6/29 16:05
 */
@Slf4j
@Service
public class WeixinPayAppService {

    @Autowired
    private WeixinPayYaml weixinPayYaml;

    @Autowired
    private WeixinPayService weixinPayService;

    /**
     * 功能描述：APP下单
     * https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_2_1.shtml
     * <p>
     * 接口返回值为，客户端拉起支付的参数
     * https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_2_4.shtml
     *
     * @param orderId      订单号
     * @param payablePrice 应付
     * @param description  商品描述
     * @param notifyUrl    通知地址
     * @return
     * @throws IOException
     */
    public WeixinPrePayResponse createOrder(String orderId, BigDecimal payablePrice, String description, String notifyUrl) throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        ObjectNode rootNode = objectMapper.createObjectNode();
        rootNode.put("appid", weixinPayYaml.getAppId())
                .put("mchid", weixinPayYaml.getMchId())
                .put("description", description)
                .put("out_trade_no", orderId)
                .put("notify_url", notifyUrl);
        rootNode.putObject("amount")
                .put("total", payablePrice.multiply(BigDecimal.valueOf(100)).toBigInteger())
                .put("currency", "CNY");

        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        objectMapper.writeValue(bos, rootNode);

        HttpPost http = new HttpPost("https://api.mch.weixin.qq.com/v3/pay/transactions/app");
        http.addHeader("Accept", "application/json");
        http.addHeader("Content-type", "application/json; charset=utf-8");
        http.setEntity(new StringEntity(bos.toString("UTF-8"), "UTF-8"));
        CloseableHttpResponse response = weixinPayService.client().execute(http);

        if (response.getStatusLine().getStatusCode() != 200) { //处理成功
            log.error("微信支付 - APP下单失败，订单号：[{}], 原因：[{}]", orderId, response.getStatusLine());
        }

        JSONObject jsonObject = JSONUtil.parseObj(EntityUtils.toString(response.getEntity()));

        String timeStamp = String.format("%s", System.currentTimeMillis() / 1000);
        String nonceStr = weixinPayService.nonceStr(32);
        StringBuilder message = new StringBuilder();
        message.append(weixinPayYaml.getAppId()).append("\n");
        message.append(timeStamp).append("\n");
        message.append(nonceStr).append("\n");
        message.append(jsonObject.getStr("prepay_id")).append("\n");
        String sign = weixinPayService.paySign(message.toString());

        /* 返回客户端拉起支付的参数 */
        WeixinPrePayResponse prePayResponse = new WeixinPrePayResponse();
        prePayResponse.setPrepayid(jsonObject.getStr("prepay_id"));
        prePayResponse.setPartnerid(weixinPayYaml.getMchId());
        prePayResponse.setAppid(weixinPayYaml.getAppId());
        prePayResponse.setTimestamp(timeStamp);
        prePayResponse.setNoncestr(nonceStr);
        prePayResponse.setSign(sign);

        return prePayResponse;
    }

    /**
     * 功能描述：根据【商户订单号】查询订单
     * https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_2_2.shtml
     *
     * @param orderId
     * @return
     * @throws Exception
     */
    public PaySuccessNotifyResponse queryOrderByOrderId(String orderId) throws Exception {
        URIBuilder uriBuilder = new URIBuilder("https://api.mch.weixin.qq.com/v3/pay/transactions/out-trade-no/" + orderId);
        uriBuilder.setParameter("mchid", weixinPayYaml.getMchId());

        HttpGet http = new HttpGet(uriBuilder.build());
        http.addHeader("Accept", "application/json");
        http.addHeader("Content-type", "application/json; charset=utf-8");
        CloseableHttpResponse response = weixinPayService.client().execute(http);

        if (response.getStatusLine().getStatusCode() != 200) { //处理成功
            log.error("微信支付 - 查询订单失败，订单号：[{}], 原因：[{}]", orderId, response.getStatusLine());
        }

        return JSONUtil.toBean(JSONUtil.parseObj(EntityUtils.toString(response.getEntity())), PaySuccessNotifyResponse.class);
    }

}
