package com.studio.nx.pay.weixin.service;

import cn.hutool.core.io.resource.ClassPathResource;
import com.studio.nx.pay.weixin.config.WeixinPayYaml;
import com.wechat.pay.contrib.apache.httpclient.WechatPayHttpClientBuilder;
import com.wechat.pay.contrib.apache.httpclient.auth.AutoUpdateCertificatesVerifier;
import com.wechat.pay.contrib.apache.httpclient.auth.PrivateKeySigner;
import com.wechat.pay.contrib.apache.httpclient.auth.WechatPay2Credentials;
import com.wechat.pay.contrib.apache.httpclient.auth.WechatPay2Validator;
import org.apache.http.impl.client.CloseableHttpClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.security.*;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;
import java.util.Random;

/**
 * 功能描述：
 * https://wechatpay-api.gitbook.io/wechatpay-api-v3
 *
 * @Author 梁展鹏
 * @Date 2021/6/29 09:10
 */
@Service
public class WeixinPayService {

    /* 生成微信支付nonce使用的字符 */
    private static final String BASE_CHAR = "qwertyuiopasdfghjklzxcvbnm1234567890";

    /* 以下static final 均为微信支付所需url,自行查看文档 */
    public static class App {
    }

    public static final String CODE_SESSION = "https://api.weixin.qq.com/sns/jscode2session?appid=%s&secret=%s&js_code=%s&grant_type=authorization_code";
    public static final String SEND_MESSAGE = "https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token=ACCESS_TOKEN";
    public static final String ACCESS_TOKEN = "https://api.weixin.qq.com/cgi-bin/token?grant_type=client_credential&appid=APPID&secret=APPSECRET";
    public static final String CREATE_CODE = "https://api.weixin.qq.com/cgi-bin/wxaapp/createwxaqrcode?access_token=ACCESS_TOKEN";
    public static final String CREATE_ORDER = "https://api.mch.weixin.qq.com/v3/pay/transactions/jsapi";
    public static final String REFUND = "https://api.mch.weixin.qq.com/v3/refund/domestic/refunds";
    public static final String QUERY_ORDER = "https://api.mch.weixin.qq.com/v3/pay/transactions/out-trade-no/{order}?mchid={mchid}";

    @Autowired
    private WeixinPayYaml weixinPayYaml;

    /**
     * 功能描述：获取私钥。
     * https://pay.weixin.qq.com/wiki/doc/apiv3/wechatpay/wechatpay7_1.shtml
     *
     * @param filename 私钥文件路径  (required)
     * @return 私钥对象
     */
    private PrivateKey getPrivateKey(String filename) {
        ClassPathResource classPathResource = new ClassPathResource(filename);
        String content = classPathResource.readUtf8Str();
        try {
            String privateKey = content
                    .replace("-----BEGIN PRIVATE KEY-----", "")
                    .replace("-----END PRIVATE KEY-----", "")
                    .replaceAll("\\s+", "");

            KeyFactory kf = KeyFactory.getInstance("RSA");
            return kf.generatePrivate(new PKCS8EncodedKeySpec(Base64.getDecoder().decode(privateKey)));
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("当前Java环境不支持RSA", e);
        } catch (InvalidKeySpecException e) {
            throw new RuntimeException("无效的密钥格式");
        }
    }

    public CloseableHttpClient client() {
        PrivateKey privateKey = getPrivateKey(weixinPayYaml.getCertPath());
        AutoUpdateCertificatesVerifier verifier = new AutoUpdateCertificatesVerifier(
                new WechatPay2Credentials(weixinPayYaml.getMchId(), new PrivateKeySigner(weixinPayYaml.getMchSerial(), privateKey)),
                weixinPayYaml.getV3Secret().getBytes(StandardCharsets.UTF_8)
        );

        WechatPayHttpClientBuilder builder = WechatPayHttpClientBuilder.create()
                .withMerchant(weixinPayYaml.getMchId(), weixinPayYaml.getMchSerial(), privateKey)
                .withValidator(new WechatPay2Validator(verifier));
        return builder.build();
    }

    public String nonceStr(int len) {
        Random random = new Random();
        StringBuilder sb = new StringBuilder();
        int length = BASE_CHAR.length();
        for (int i = 0; i < len; i++) {
            int number = random.nextInt(length);
            sb.append(BASE_CHAR.charAt(number));
        }
        return sb.toString();
    }

    public String paySign(String message) throws NoSuchAlgorithmException, InvalidKeyException, SignatureException {
        Signature sign = Signature.getInstance("SHA256withRSA");
        sign.initSign(getPrivateKey(weixinPayYaml.getCertPath()));
        sign.update(message.getBytes(StandardCharsets.UTF_8));
        return Base64.getEncoder().encodeToString(sign.sign());
    }

}
