package com.studio.nx.yxl.controller.app;

import com.alibaba.druid.util.StringUtils;
import com.google.gson.Gson;
import com.studio.nx.framework.dto.req.BaseRequest;
import com.studio.nx.framework.dto.resp.BaseResponse;
import com.studio.nx.framework.jwt.annotation.IgnoreAuth;
import com.studio.nx.framework.jwt.util.JWTUtil;
import com.studio.nx.pay.weixin.config.WeixinPayYaml;
import com.studio.nx.pay.weixin.dto.NotifyBody;
import com.studio.nx.pay.weixin.dto.NotifyResult;
import com.studio.nx.pay.weixin.dto.PaySuccessNotifyResponse;
import com.studio.nx.pay.weixin.dto.WeixinPrePayResponse;
import com.studio.nx.pay.weixin.service.WeixinPayAppService;
import com.studio.nx.pay.weixin.service.WeixinPayService;
import com.studio.nx.yxl.controller.app.response.order.OrderResponse;
import com.studio.nx.yxl.service.OrderService;
import com.wechat.pay.contrib.apache.httpclient.util.AesUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.nio.charset.StandardCharsets;

/**
 * 功能描述：
 * 微信支付 - 商户平台
 * https://pay.weixin.qq.com/wiki/doc/apiv3/index.shtml
 * <p>
 * APP支付
 * https://pay.weixin.qq.com/wiki/doc/apiv3/open/pay/chapter2_5_0.shtml
 */
@Api(tags = "支付服务 - 微信")
@Slf4j
@Validated
@RestController("AppPayWeixinController")
@RequestMapping("/app/pay/weixin")
public class WeixinPayController {

    @Value("${spring.profiles.active}")
    private String active;

    @Autowired
    private WeixinPayYaml weixinPayYaml;

    @Autowired
    private WeixinPayService weixinPayService;

    @Autowired
    private WeixinPayAppService weixinPayAppService;

    @Autowired
    private OrderService orderService;

    @ApiOperation(value = "微信支付 - 预支付")
    @PostMapping("/{orderId:\\d+}/pre-pay")
    public ResponseEntity<BaseResponse<WeixinPrePayResponse>> wxPrePay(
            @ApiParam(value = "订单标识") @NotBlank @PathVariable("orderId") String orderId,
            @Valid @RequestBody BaseRequest request) throws Exception {
        String accountId = JWTUtil.getLocalUser().get().getAccountId();
        OrderResponse orderResponse = orderService.findByMyOrder(orderId, accountId);

        if (orderResponse.getOrderItemResponseList().size() <=0 ) {
            return ResponseEntity.ok(BaseResponse.failure(HttpStatus.INTERNAL_SERVER_ERROR.value(), "生成订单异常"));
        }

        String description = String.format("购买 %s 张照片", orderResponse.getOrderItemResponseList().size());
        return ResponseEntity.ok(BaseResponse.ok(weixinPayAppService.createOrder(orderId, orderResponse.getPayablePrice(), description, getPayWeixinNotifyUrl())));
    }

    @ApiOperation(value = "微信支付 - 客户端轮训查询订单")
    @GetMapping("/{orderId:\\d+}")
    public ResponseEntity<BaseResponse<?>> queryOrder(@ApiParam(value = "订单标识") @NotBlank @PathVariable("orderId") String orderId) throws Exception {
        Boolean flag = orderService.queryOrderByOrderId(orderId);
        return ResponseEntity.ok(flag ? BaseResponse.ok() : BaseResponse.failure(HttpStatus.INTERNAL_SERVER_ERROR.value(), "支付异常"));
    }

    /**
     * 功能描述：微信支付 - 支付回调通知
     * todo 消息队列解藕
     * 支付结果通知：
     * https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_2_5.shtml
     * https://pay.weixin.qq.com/wiki/doc/api/wxa/wxa_api.php?chapter=9_7&index=8
     * <p>
     * 回调通知注意事项：
     * https://pay.weixin.qq.com/wiki/doc/api/wxa/wxa_api.php?chapter=23_8&index=5
     *
     * @param request
     * @param response
     * @return
     * @throws Exception
     */
    @IgnoreAuth
//    @ApiOperation(value = "微信支付 - 支付回调通知")
    @PostMapping("/notify")
    public NotifyResult wxNotify(@RequestBody NotifyBody body, HttpServletRequest request, HttpServletResponse response) throws Exception {
        if (!StringUtils.equals("TRANSACTION.SUCCESS", body.getEvent_type())) {
//            [NotifyBody(
//                id=ffd0916c-8517-565e-ad9c-4d214b7d2ca3,
//                create_time=2021-06-29T23:29:00+08:00,
//                event_type=TRANSACTION.SUCCESS,
//                resource_type=encrypt-resource,
//                summary=支付成功,
//                resource=NotifyBody.Resource(
//                    algorithm=AEAD_AES_256_GCM,
//                    ciphertext=,
//                    associated_data=transaction,
//                    original_type=transaction,
//                    nonce=RxGFDVgXh8XA
//                )
//            )]
            log.error("微信支付 - 回调，支付异常。参数：[{}]", body.toString());
            return NotifyResult.create().success();
        }

        /* 通知参数解密 */
        NotifyBody.Resource resource = body.getResource();
        String ret = new AesUtil(weixinPayYaml.getV3Secret().getBytes(StandardCharsets.UTF_8))
                .decryptToString(resource.getAssociated_data().getBytes(StandardCharsets.UTF_8), resource.getNonce().getBytes(StandardCharsets.UTF_8), resource.getCiphertext());

//        [{
//            "mchid": "1610957462",
//            "appid": "wx05293a86e894a20d",
//            "out_trade_no": "1409527396216246274",
//            "transaction_id": "4200001152202106292898306570",
//            "trade_type": "APP",
//            "trade_state": "SUCCESS",
//            "trade_state_desc": "支付成功",
//            "bank_type": "CCB_DEBIT",
//            "attach": "",
//            "success_time": "2021-06-29T23:29:00+08:00",
//            "payer": {
//                "openid": "oLXdh6oLFM3zvERNdoku58xskCiE"
//            },
//            "amount": {
//                "total": 1,
//                "payer_total": 1,
//                "currency": "CNY",
//                "payer_currency": "CNY"
//            }
//        }]
        log.info("微信支付[回调] 解密：[{}]", ret);
        PaySuccessNotifyResponse payResponse = new Gson().fromJson(ret, PaySuccessNotifyResponse.class);
        if (!StringUtils.equals(payResponse.getTrade_state(), "SUCCESS")) {
            log.error("微信支付 - 回调，解密支付异常。参数：[{}]", payResponse.toString());
            return NotifyResult.create().success();
        }

        orderService.paySuccessToEnd(payResponse);

        return NotifyResult.create().success();
    }

    // todo 还缺一个退款通知的金额统计... https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_2_11.shtml

    private String getPayWeixinNotifyUrl() {
        if ("prod".equals(active)) {
            return "https://youxiangle.cn/yxl-back-end/app/pay/weixin/notify";
        } else {
            return "https://430jy.uutime.cn/yxl-back-end/app/pay/weixin/notify";
        }
    }

}
