package com.studio.nx.yxl.service;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import com.aliyun.oss.model.ObjectMetadata;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.oss.alioss.service.AliOssService;
import com.studio.nx.oss.dto.FileResponse;
import com.studio.nx.oss.local.config.LocalYaml;
import com.studio.nx.oss.local.service.LocalService;
import com.studio.nx.oss.util.MetadataUtil;
import com.studio.nx.yxl.constant.OssConstant;
import com.studio.nx.yxl.entity.Activity;
import com.studio.nx.yxl.entity.Kindergarten;
import com.studio.nx.yxl.entity.PhotoWarehouse;
import com.studio.nx.yxl.mapper.ActivityMapper;
import com.studio.nx.yxl.mapper.PhotoWarehouseMapper;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import net.coobird.thumbnailator.Thumbnails;
import net.coobird.thumbnailator.geometry.Positions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;
import org.springframework.util.StringUtils;

import javax.imageio.ImageIO;
import javax.transaction.Transactional;
import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Optional;

/**
 * 功能描述：
 * 1. 摄影师通过管理后台上传照片到Minio（@see com.studio.nx.oss.minio.controller.MinioController#upload）
 * 2. 把文件保存到Minio的信息提交保存图片库（@see com.studio.nx.yxl.controller.admin.PhotoWarehouseController#save）
 * 3. 运营人员通过管理后台审核上传图片是否符合要求（@see com.studio.nx.yxl.controller.admin.PhotoWarehouseController#audit）
 * 4. 定时任务扫描【t_photo_warehouse】已通过审核的，还没处理的数据，
 * 5. 下载【远程原图】到本地的临时目录
 * 6. 进行裂变（预览图、各个尺寸的水印logo图）
 * 6.1 列表预览图（压缩小图，幼儿园logo、品牌logo）；上传至Minio；并更新表【t_photo_warehouse】字段
 * 6.2 详情页预览图（压缩小图，水印、幼儿园logo、品牌logo）；上传至Minio；并更新表【t_photo_warehouse】字段
 * 6.3 购买到高清图（压缩图，幼儿园logo、品牌logo；）；上传至Minio；并更新表【t_photo_warehouse】字段
 * 7. 更新表【t_photo_warehouse】数据设置为已处理
 * 8. 删除本地临时目录
 *
 * @Author 梁展鹏
 * @Date 2021/6/12 22:29
 */
@Slf4j
@Service
public class PhotoWarehouseProcessingService {

    @Value("${spring.profiles.active}")
    private String active;

    @Autowired
    private LocalYaml localYaml;

    @Autowired
    private LocalService localService;

    @Autowired
    private AliOssService aliOssService;

    @Autowired
    private PhotoWarehouseMapper mapper;

    @Autowired
    private KindergartenService kindergartenService;

    @Autowired
    private ActivityMapper activityMapper;

    private static final String LOCAL_TEMP_PATH = "/tmp"; // 文件保存到本地临时目录
    private final static String PHOTO_PIXEL_FORMAT = "%s x %s"; // 图片分辨率的格式

    /* 全局水印的透明度 */
    private final static Float WATERMARK_OPACITY = 1F;

    /* 列表预览图 */
    private final static String PHOTO_PREVIEW_LIST_LOCAL_PATH = "/kindergarten/%s/%s_%s_%.0f_list.%s"; // 本地路径
    private final static Float PHOTO_PREVIEW_LIST_QUALITY = 0.5F; // 压缩质量
    private final static Integer PHOTO_PREVIEW_LIST_WIDTH = 800; // 宽度
    private final static Integer PHOTO_PREVIEW_LIST_HEIGHT = 600; // 高度

    /* 详情页预览图 */
    private final static String PHOTO_PREVIEW_DETAIL_LOCAL_PATH = "/kindergarten/%s/%s_%s_%.0f_detail.%s"; // 本地路径
    private final static Float PHOTO_PREVIEW_DETAIL_QUALITY = 1F; // 压缩质量
    private final static Integer PHOTO_PREVIEW_DETAIL_WIDTH = 800; // 宽度
    private final static Integer PHOTO_PREVIEW_DETAIL_HEIGHT = 600; // 高度

    /* 高清图的压缩尺寸、质量 */
    private final static String PHOTO_HD_LOCAL_PATH = "%s/%s_%s_%.0f_hd_logo.%s"; // 本地路径
    private final static Float PHOTO_HD_QUALITY = 1F; // 压缩质量
    private final static Integer PHOTO_HD_WIDTH = 3000; // 宽度
    private final static Integer PHOTO_HD_HEIGHT = 2000; // 高度

    /**
     * 功能描述：入口方法
     */
    @Async
    public void start() {
        StopWatch watch = new StopWatch();
        watch.start();

        /* 4. 定时任务扫描【t_photo_warehouse】已通过审核的，还没处理的数据， */
        List<PhotoWarehouse> list = toBeProcessed(0, 250); // 每次取10条记录处理

        /* 逐张图片进行一一处理 */
        for (PhotoWarehouse photoWarehouse : list) {
            oneByOne(photoWarehouse);
        }

        watch.stop();
        log.info("start in [{}] ms", watch.getTotalTimeMillis());
    }

    /**
     * 功能描述：在上传照片后进行预览图裂变动作
     */
    @Async
    @Transactional
    public void before(PhotoWarehouse photoWarehouse) {
        String downloadTempPath = "";
        try {
            /* 5. 下载【远程原图】到本地的临时目录 */
            downloadTempPath = downloadOriginPhoto(photoWarehouse);

            /* 更新原图的Metadata信息 */
            updateByOriginPhotoMetadata(photoWarehouse, downloadTempPath);

            /* 6. 进行裂变（预览图、各个尺寸的水印logo图） */
            /* 6.1 列表预览图（压缩小图，幼儿园logo、品牌logo）；上传至Minio；并更新表【t_photo_warehouse】字段 */
            updateByListPreview(photoWarehouse, downloadTempPath);

            /* 6.2 详情页预览图（压缩小图，水印、幼儿园logo、品牌logo）；上传至Minio；并更新表【t_photo_warehouse】字段 */
            updateByDetailPreview(photoWarehouse, downloadTempPath);
        } catch (Exception e) {
            log.error("裂变图片出错，[{}], [{}]", e.getMessage(), e);
        } finally {
            /* 8. 删除本地临时目录 */
            FileUtil.del(new File(downloadTempPath).getParent());
        }
    }

    @SneakyThrows
    @Async
    @Transactional
    public void oneByOne(PhotoWarehouse photoWarehouse) {
        // add by 梁展鹏 20210811 故意让线程随机睡眠一小会
        Thread.sleep(RandomUtil.randomInt(2000));

        String downloadTempPath = "";
        try {
            /* 5. 下载【远程原图】到本地的临时目录 */
            downloadTempPath = downloadOriginPhoto(photoWarehouse);

            /* 更新原图的Metadata信息（原图分辨率、拍摄时间、是否横图） */
//            if (!StringUtils.hasText(photoWarehouse.getOriginPhotoPixel())
//                    || null == photoWarehouse.getPhotoTime()
//                    || null == photoWarehouse.getIsHorizontal()
//            ) {
                updateByOriginPhotoMetadata(photoWarehouse, downloadTempPath);
//            }

            /* 6. 进行裂变（预览图、各个尺寸的水印logo图） */
            /* 6.1 列表预览图（压缩小图，幼儿园logo、品牌logo）；上传至Minio；并更新表【t_photo_warehouse】字段 */
//            if (!StringUtils.hasText(photoWarehouse.getListPreview()) || !localService.isExists(photoWarehouse.getListPreview())) {
                updateByListPreview(photoWarehouse, downloadTempPath);
//            }

            /* 6.2 详情页预览图（压缩小图，水印、幼儿园logo、品牌logo）；上传至Minio；并更新表【t_photo_warehouse】字段 */
//            if (!StringUtils.hasText(photoWarehouse.getDetailPreview()) || !localService.isExists(photoWarehouse.getDetailPreview())) {
                updateByDetailPreview(photoWarehouse, downloadTempPath);
//            }

            /* 6.3 购买到高清图（压缩图，幼儿园logo、品牌logo；）；上传至Minio；并更新表【t_photo_warehouse】字段 */
//            if (!StringUtils.hasText(photoWarehouse.getHdLogoPhoto()) || !minioService.isExists(photoWarehouse.getHdLogoPhoto())) {
                updateByHdLogoPhoto(photoWarehouse, downloadTempPath);
//            }

            /* 7. 更新表【t_photo_warehouse】数据设置为已处理 */
            isReady(photoWarehouse.getId());
        } catch (Exception e) {
            log.error("裂变图片出错，照片id：[{}]，照片源路径：[{}], [{}], [{}]", photoWarehouse.getId(), photoWarehouse.getOriginPhoto(), e.getMessage(), e);
        } finally {
            /* 8. 删除本地临时目录 */
            FileUtil.del(new File(downloadTempPath).getParent());
        }
    }

    /**
     * 功能描述：分页查询待处理的数据（审核通过、还没处理）
     *
     * @param page
     * @param size
     * @return
     */
    public List<PhotoWarehouse> toBeProcessed(Integer page, Integer size) {
        return mapper.selectPage(new Page(page, size), new LambdaQueryWrapper<PhotoWarehouse>()
                .eq(PhotoWarehouse::getAuditStatus, PhotoWarehouse.AuditStatus.PASS) // 审核通过
                .eq(PhotoWarehouse::getIsReady, Boolean.FALSE) // 待裂变
                .orderByAsc(PhotoWarehouse::getAuditDate)
        ).getRecords();
    }

    /**
     * 功能描述：下载原图到本地临时目录
     *
     * @param photoWarehouse
     */
    private String downloadOriginPhoto(PhotoWarehouse photoWarehouse) {
        String filePath = photoWarehouse.getOriginPhoto();
        String filename = filePath.substring(filePath.lastIndexOf("/") + 1);
        String downloadPath = localYaml.getPath() + LOCAL_TEMP_PATH + File.separator + IdUtil.simpleUUID() + File.separator + filename;

        aliOssService.download(OssConstant.YXL_KINDERGARTEN.BUCKETNAME, filePath, downloadPath);

        /* add by 梁展鹏 20210902 如果读取照片的元数据为Apple则走OSS的图片预处理，格式化为jpg格式 */
        if (isiOSHeic(downloadPath)) {
            aliOssService.processAndDownload(OssConstant.YXL_KINDERGARTEN.BUCKETNAME, filePath, downloadPath);
        }

        return downloadPath;
    }

    /**
     * 功能描述：更新原图的Metadata信息
     *
     * @param photoWarehouse
     * @param downloadTempPath 本地目录
     */
    private void updateByOriginPhotoMetadata(PhotoWarehouse photoWarehouse, String downloadTempPath) {
        MetadataUtil.ImageMetadata imageMetadata = MetadataUtil.parseImgInfo(downloadTempPath);
        if (!Optional.ofNullable(imageMetadata).isPresent()) {
            return;
        }

        /* 照片拍摄时间 */
        LocalDateTime photoTime = null;
        if (StringUtils.hasText(imageMetadata.getDateTime())) {
            photoTime = LocalDateTime.parse(imageMetadata.getDateTime(), DateTimeFormatter.ofPattern("yyyy:MM:dd HH:mm:ss"));
        } else {
            /* 上传至Minio之后，再下载就丢失拍摄时间元数据。如果照片的元数据拿不到，则用照片的上传拍照时间 */
            log.error("读取照片的拍摄时间错误，[{}]", photoWarehouse.getId());
            photoTime = photoWarehouse.getCreateAt();
        }

        /* 分辨率格式化 */
        String originPhotoPixel = getPhotoPixel(imageMetadata);

        /* 根据分辨率判断图片横图/竖图 */
        Integer width = getPhotoPixelWidth(originPhotoPixel);
        Integer height = getPhotoPixelHeight(originPhotoPixel);
        Boolean isHorizontal = Boolean.TRUE;
        if (width < height) {
            isHorizontal = Boolean.FALSE;
        }
        photoWarehouse.setIsHorizontal(isHorizontal);

        /* 更新元数据 */
        mapper.update(null, new LambdaUpdateWrapper<PhotoWarehouse>()
                .eq(PhotoWarehouse::getId, photoWarehouse.getId())
                .set(PhotoWarehouse::getUpdateAt, LocalDateTime.now())
                .set(PhotoWarehouse::getPhotoTime, photoTime)
                .set(PhotoWarehouse::getOriginPhotoPixel, originPhotoPixel)
                .set(PhotoWarehouse::getIsHorizontal, isHorizontal)
        );
    }

    /**
     * 功能描述：更新列表预览图
     *
     * @param photoWarehouse
     * @param downloadTempPath
     */
    @SneakyThrows
    private void updateByListPreview(PhotoWarehouse photoWarehouse, String downloadTempPath) {
        String listPreviewPath = "";
        if (photoWarehouse.getIsHorizontal()) {
            listPreviewPath = String.format(PHOTO_PREVIEW_LIST_LOCAL_PATH,
                    getBucketName(photoWarehouse),
                    FileUtil.mainName(downloadTempPath),
                    String.format("%sx%s", PHOTO_PREVIEW_LIST_WIDTH, PHOTO_PREVIEW_LIST_HEIGHT),
                    PHOTO_PREVIEW_LIST_QUALITY * 100,
                    FileUtil.extName(downloadTempPath)
            );
        } else {
            listPreviewPath = String.format(PHOTO_PREVIEW_LIST_LOCAL_PATH,
                    getBucketName(photoWarehouse),
                    FileUtil.mainName(downloadTempPath),
                    String.format("%sx%s", PHOTO_PREVIEW_LIST_HEIGHT, PHOTO_PREVIEW_LIST_WIDTH),
                    PHOTO_PREVIEW_LIST_QUALITY * 100,
                    FileUtil.extName(downloadTempPath)
            );
        }
        FileUtil.mkdir(new File(localYaml.getPath() + listPreviewPath).getParent());

        Thumbnails.Builder<File> thumbnails = Thumbnails.of(downloadTempPath);
        if (photoWarehouse.getIsHorizontal()) {
            thumbnails.size(PHOTO_PREVIEW_LIST_WIDTH, PHOTO_PREVIEW_LIST_HEIGHT);
        } else {
            thumbnails.size(PHOTO_PREVIEW_LIST_HEIGHT, PHOTO_PREVIEW_LIST_WIDTH);
        }

        /* 水印 */
        if (StringUtils.hasText(photoWarehouse.getActivityId())) {
            Activity activity = activityMapper.selectById(photoWarehouse.getActivityId());
            String activityLogoPath = getActivityLogo(photoWarehouse.getActivityId(), Boolean.FALSE);
            if (StringUtils.hasText(activity.getPreviewLogo()) && StringUtils.hasText(activityLogoPath)) {
                thumbnails.watermark(Positions.TOP_LEFT, ImageIO.read(new File(activityLogoPath)), WATERMARK_OPACITY); // 活动logo
            }
        }
        thumbnails.watermark(Positions.BOTTOM_LEFT, ImageIO.read(getYxlLogo(Boolean.FALSE)), WATERMARK_OPACITY); // 品牌logo
        thumbnails.watermark(Positions.BOTTOM_RIGHT, ImageIO.read(new File(getKindergartenLogo(photoWarehouse.getKindergartenId(), Boolean.FALSE))), WATERMARK_OPACITY); // 幼儿园logo
        thumbnails.outputQuality(PHOTO_PREVIEW_LIST_QUALITY);
        thumbnails.toFile(localYaml.getPath() + listPreviewPath);

        mapper.update(null, new LambdaUpdateWrapper<PhotoWarehouse>()
                .eq(PhotoWarehouse::getId, photoWarehouse.getId())
                .set(PhotoWarehouse::getUpdateAt, LocalDateTime.now())
                .set(PhotoWarehouse::getListPreview, listPreviewPath)
        );
    }

    /**
     * 功能描述：更新详情页预览图
     *
     * @param photoWarehouse
     * @param downloadTempPath
     */
    @SneakyThrows
    private void updateByDetailPreview(PhotoWarehouse photoWarehouse, String downloadTempPath) {
        String detailPreviewPath = "";
        if (photoWarehouse.getIsHorizontal()) {
            detailPreviewPath = String.format(PHOTO_PREVIEW_DETAIL_LOCAL_PATH,
                    getBucketName(photoWarehouse),
                    FileUtil.mainName(downloadTempPath),
                    String.format("%sx%s", PHOTO_PREVIEW_LIST_WIDTH, PHOTO_PREVIEW_LIST_HEIGHT),
                    PHOTO_PREVIEW_DETAIL_QUALITY * 100,
                    FileUtil.extName(downloadTempPath)
            );
        } else {
            detailPreviewPath = String.format(PHOTO_PREVIEW_DETAIL_LOCAL_PATH,
                    getBucketName(photoWarehouse),
                    FileUtil.mainName(downloadTempPath),
                    String.format("%sx%s", PHOTO_PREVIEW_LIST_HEIGHT, PHOTO_PREVIEW_LIST_WIDTH),
                    PHOTO_PREVIEW_DETAIL_QUALITY * 100,
                    FileUtil.extName(downloadTempPath)
            );
        }
        FileUtil.mkdir(new File(localYaml.getPath() + detailPreviewPath).getParent());

        Thumbnails.Builder<File> thumbnails = Thumbnails.of(downloadTempPath);
        if (photoWarehouse.getIsHorizontal()) {
            thumbnails.size(PHOTO_PREVIEW_DETAIL_WIDTH, PHOTO_PREVIEW_DETAIL_HEIGHT);
        } else {
            thumbnails.size(PHOTO_PREVIEW_DETAIL_HEIGHT, PHOTO_PREVIEW_DETAIL_WIDTH);
        }

        /* 水印 */
        thumbnails.watermark(Positions.CENTER, ImageIO.read(getYxlWatermark()), WATERMARK_OPACITY); // 品牌水印
        if (StringUtils.hasText(photoWarehouse.getActivityId())) {
            Activity activity = activityMapper.selectById(photoWarehouse.getActivityId());
            String activityLogoPath = getActivityLogo(photoWarehouse.getActivityId(), Boolean.FALSE);
            if (StringUtils.hasText(activity.getPreviewLogo()) && StringUtils.hasText(activityLogoPath)) {
                thumbnails.watermark(Positions.TOP_LEFT, ImageIO.read(new File(activityLogoPath)), WATERMARK_OPACITY); // 活动logo
            }
        }
        thumbnails.watermark(Positions.BOTTOM_LEFT, ImageIO.read(getYxlLogo(Boolean.FALSE)), WATERMARK_OPACITY); // 品牌logo
        thumbnails.watermark(Positions.BOTTOM_RIGHT, ImageIO.read(new File(getKindergartenLogo(photoWarehouse.getKindergartenId(), Boolean.FALSE))), WATERMARK_OPACITY); // 幼儿园logo
        thumbnails.outputQuality(PHOTO_PREVIEW_DETAIL_QUALITY);
        thumbnails.toFile(localYaml.getPath() + detailPreviewPath);

        mapper.update(null, new LambdaUpdateWrapper<PhotoWarehouse>()
                .eq(PhotoWarehouse::getId, photoWarehouse.getId())
                .set(PhotoWarehouse::getUpdateAt, LocalDateTime.now())
                .set(PhotoWarehouse::getDetailPreview, detailPreviewPath)
        );
    }

    /**
     * 功能描述：更新高清图
     *
     * @param photoWarehouse
     * @param downloadTempPath
     */
    @SneakyThrows
    private void updateByHdLogoPhoto(PhotoWarehouse photoWarehouse, String downloadTempPath) {
        String hdLogoPath = "";
        if (photoWarehouse.getIsHorizontal()) {
            hdLogoPath = String.format(PHOTO_HD_LOCAL_PATH,
                    new File(downloadTempPath).getParent(),
                    FileUtil.mainName(downloadTempPath),
                    String.format("%sx%s", PHOTO_HD_WIDTH, PHOTO_HD_HEIGHT),
                    PHOTO_HD_QUALITY * 100,
                    FileUtil.extName(downloadTempPath)
            );
        } else {
            hdLogoPath = String.format(PHOTO_HD_LOCAL_PATH,
                    new File(downloadTempPath).getParent(),
                    FileUtil.mainName(downloadTempPath),
                    String.format("%sx%s", PHOTO_HD_HEIGHT, PHOTO_HD_WIDTH),
                    PHOTO_HD_QUALITY * 100,
                    FileUtil.extName(downloadTempPath)
            );
        }

        Thumbnails.Builder<File> thumbnails = Thumbnails.of(downloadTempPath);
        if (photoWarehouse.getIsHorizontal()) {
            thumbnails.size(PHOTO_HD_WIDTH, PHOTO_HD_HEIGHT);
        } else {
            thumbnails.size(PHOTO_HD_HEIGHT, PHOTO_HD_WIDTH);
        }

        /* 水印 */
        if (StringUtils.hasText(photoWarehouse.getActivityId())) {
            Activity activity = activityMapper.selectById(photoWarehouse.getActivityId());
            String activityLogoPath = getActivityLogo(photoWarehouse.getActivityId(), Boolean.TRUE);
            if (StringUtils.hasText(activity.getHdLogo()) && StringUtils.hasText(activityLogoPath)) {
                thumbnails.watermark(Positions.TOP_LEFT, ImageIO.read(new File(activityLogoPath)), WATERMARK_OPACITY); // 活动logo
            }
        }
        thumbnails.watermark(Positions.BOTTOM_LEFT, ImageIO.read(getYxlLogo(Boolean.TRUE)), WATERMARK_OPACITY); // 品牌logo
        thumbnails.watermark(Positions.BOTTOM_RIGHT, ImageIO.read(new File(getKindergartenLogo(photoWarehouse.getKindergartenId(), Boolean.TRUE))), WATERMARK_OPACITY); // 幼儿园logo
        thumbnails.outputQuality(PHOTO_HD_QUALITY);
        thumbnails.toFile(hdLogoPath);

        /* 读取照片的exif信息 */
        Long hdLogoPhotoSize = 0L;
        String hdLogoPhotoPixel = "";
        MetadataUtil.ImageMetadata imageMetadata = MetadataUtil.parseImgInfo(hdLogoPath);
        if (Optional.ofNullable(imageMetadata).isPresent()) {
            hdLogoPhotoSize = imageMetadata.getSize();
            hdLogoPhotoPixel = getPhotoPixel(imageMetadata);
        }

        /* 把高清图上传至Minio */
        String bucketName = getBucketName(photoWarehouse);
        // add by 梁展鹏 20210812 判断生产模式才会执行上传文件
        Assert.isTrue("prod".equals(active));
        FileResponse fileResponse = aliOssService.putObject(OssConstant.YXL_KINDERGARTEN.BUCKETNAME, bucketName, hdLogoPath, new ObjectMetadata());

        mapper.update(null, new LambdaUpdateWrapper<PhotoWarehouse>()
                .eq(PhotoWarehouse::getId, photoWarehouse.getId())
                .set(PhotoWarehouse::getUpdateAt, LocalDateTime.now())
                .set(PhotoWarehouse::getHdLogoPhotoSize, hdLogoPhotoSize)
                .set(PhotoWarehouse::getHdLogoPhotoPixel, hdLogoPhotoPixel)
                .set(PhotoWarehouse::getHdLogoPhoto, new URI(fileResponse.getFullPath()).getPath())
        );
    }

    /**
     * 功能描述：处理完成
     *
     * @param photoWarehouseId
     */
    public void isReady(String photoWarehouseId) {
        mapper.update(null, new LambdaUpdateWrapper<PhotoWarehouse>()
                .eq(PhotoWarehouse::getId, photoWarehouseId)
                .set(PhotoWarehouse::getUpdateAt, LocalDateTime.now())
                .set(PhotoWarehouse::getIsReady, Boolean.TRUE)
        );
    }

    private String getBucketName(PhotoWarehouse photoWarehouse) {
        String bucketName = "";
        if (photoWarehouse.getIsTake()) { // 随手拍
            bucketName = String.format("%s-%s", photoWarehouse.getKindergartenId(), "take");
        } else if (StringUtils.hasText(photoWarehouse.getActivityId())) {
            bucketName = String.format("%s-%s", photoWarehouse.getKindergartenId(), photoWarehouse.getActivityId());
        } else {
            bucketName = photoWarehouse.getKindergartenId();
        }

        return bucketName;
    }

    private Boolean isiOSHeic(String downloadTempPath) {
        MetadataUtil.ImageMetadata imageMetadata = MetadataUtil.parseImgInfo(downloadTempPath);
        return "apple".equalsIgnoreCase(imageMetadata.getModel()) ? Boolean.TRUE : Boolean.FALSE;
    }

    /**
     * 功能描述：获取图片的分辨率
     *
     * @param downloadTempPath
     * @return
     */
    private String getOriginPhotoPixel(String downloadTempPath) {
        MetadataUtil.ImageMetadata imageMetadata = MetadataUtil.parseImgInfo(downloadTempPath);
        if (Optional.ofNullable(imageMetadata).isPresent()) {
            /* 分辨率格式化 */
            return getPhotoPixel(imageMetadata);
        }

        return String.format(PHOTO_PIXEL_FORMAT, 800, 600);
    }

    /**
     * 功能描述：根据图片的分辨率获取宽度
     *
     * @param photoPixel
     * @return
     */
    private Integer getPhotoPixelWidth(String photoPixel) {
        Integer width = 0;
        try {
            width = Integer.parseInt(photoPixel.split(" x ")[0]);
        } catch (Exception e) {
            log.error("获取图片宽度失败：{}", e);
            width = 800;
        } finally {
            return width;
        }
    }

    /**
     * 功能描述：根据图片的分辨率获取高度
     *
     * @param photoPixel
     * @return
     */
    private Integer getPhotoPixelHeight(String photoPixel) {
        Integer height = 0;
        try {
            height = Integer.parseInt(photoPixel.split(" x ")[1]);
        } catch (Exception e) {
            log.error("获取图片高度失败：{}", e);
            height = 600;
        } finally {
            return height;
        }
    }

    /**
     * 功能描述：格式化图片的分辨率。返回（宽度 x 高度）
     *
     * @param imageMetadata
     * @return
     */
    private String getPhotoPixel(MetadataUtil.ImageMetadata imageMetadata) {
        if (!StringUtils.hasText(imageMetadata.getExifImageWidth()) || !StringUtils.hasText(imageMetadata.getExifImagHeight())) {
            return String.format(PHOTO_PIXEL_FORMAT, imageMetadata.getImgWidth().split(" pixels")[0], imageMetadata.getImgHeight().split(" pixels")[0]);
        } else {
            return String.format(PHOTO_PIXEL_FORMAT, imageMetadata.getExifImageWidth(), imageMetadata.getExifImagHeight());
        }
    }

    @SneakyThrows
    private InputStream getYxlLogo(Boolean isBig) {
        ClassPathResource resource;
        if (isBig) {
            resource = new ClassPathResource("static/yxl/fission/hd_logo.png");
        } else {
            resource = new ClassPathResource("static/yxl/fission/preview_logo.png");
        }

        if (!resource.exists()) {
            log.error("幼享乐 logo 不存在：[{}]", resource.getPath());
            throw new BusinessException("[幼享乐]logo不存在");
        }

        return resource.getInputStream();
    }

    @SneakyThrows
    private InputStream getYxlWatermark() {
        ClassPathResource resource = new ClassPathResource("static/yxl/fission/preview_watermark.png");
        if (!resource.exists()) {
            log.error("幼享乐 watermark 不存在：[{}]", resource.getPath());
            throw new BusinessException("[幼享乐]watermark不存在");
        }

        return resource.getInputStream();
    }

    @SneakyThrows
    private String getKindergartenLogo(String kindergartenId, Boolean isHdLogo) {
        Optional<Kindergarten> optional = kindergartenService.findById(kindergartenId);
        if (!optional.isPresent()) {
            throw new BusinessException("数据错误");
        }

        /* 预览图取小logo，高清图取大logo */
        String path;
        if (isHdLogo) {
            path = localYaml.getPath() + optional.get().getHdLogo();
        } else {
            path = localYaml.getPath() + optional.get().getPreviewLogo();
        }
        File file = new File(path);
        if (!file.exists()) {
            log.error("幼儿园 logo 不存在：kindergartenId: [{}], path: [{}]", kindergartenId, path);
            throw new BusinessException("[幼儿园]logo不存在");
        }

        return path;
    }

    @SneakyThrows
    private String getActivityLogo(String activityId, Boolean isHdLogo) {
        Activity _entity = activityMapper.selectById(activityId);
        if (!Optional.ofNullable(_entity).isPresent()) {
            throw new BusinessException("数据错误");
        }

        /* 预览图取小logo，高清图取大logo */
        String path;
        if (isHdLogo && StringUtils.hasText(_entity.getHdLogo())) {
            path = localYaml.getPath() + _entity.getHdLogo();
        } else if (!isHdLogo && StringUtils.hasText(_entity.getPreviewLogo())) {
            path = localYaml.getPath() + _entity.getPreviewLogo();
        } else {
            return "";
        }

        File file = new File(path);
        if (!file.exists()) {
            log.error("活动 logo 不存在：activityId：[{}], paht：[{}]", activityId, path);
            throw new BusinessException("[活动]logo不存在");
        }

        return path;
    }

}
