package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.oss.minio.service.MinioService;
import com.studio.nx.yxl.controller.app.request.my.apply.PhotoerApplySaveRequest;
import com.studio.nx.yxl.controller.app.response.my.applys.PhotoerApplySaveResponse;
import com.studio.nx.yxl.entity.PhotoerAudit;
import com.studio.nx.yxl.entity.PhotoerAuditMapStruct;
import com.studio.nx.yxl.mapper.PhotoerAuditMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.transaction.Transactional;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

/**
 * 功能描述：摄影师认证
 *
 * @Author 梁展鹏
 * @Date 2021/6/17 16:18
 */
@Service
public class PhotoerApplyService {

    @Autowired
    private MinioService minioService;

    @Autowired
    private PhotoerAuditMapper photoerAuditMapper;

    public static final String PHOTOER_APPLY_BUCKET_NAME = "photoer-apply";

    public Optional<PhotoerAudit> findByAccountId(String accountId) {
        return Optional.ofNullable(
                photoerAuditMapper.selectOne(new LambdaQueryWrapper<PhotoerAudit>()
                        .eq(PhotoerAudit::getAccountId, accountId)
                        .eq(PhotoerAudit::getIsDel, Boolean.FALSE)
                )
        );
    }

    public PhotoerApplySaveResponse photoerGet(String accountId) {
        Optional<PhotoerAudit> optional = findByAccountId(accountId);
        /* 不存在时返回空 */
        if (!optional.isPresent()) {
            return null;
        }

        PhotoerAudit _entity = optional.get();
        PhotoerApplySaveResponse dto = PhotoerAuditMapStruct.INSTANCES.toResponse(_entity);
        if (StringUtils.hasText(dto.getIdCardFront())) {
            dto.setIdCardFront(minioService.getPresignedObjectUrl(PHOTOER_APPLY_BUCKET_NAME, minioService.cutFilename(PHOTOER_APPLY_BUCKET_NAME, dto.getIdCardFront()), 5, TimeUnit.MINUTES)); // 申请人证件正面
        }
        if (StringUtils.hasText(dto.getIdCardBack())) {
            dto.setIdCardBack(minioService.getPresignedObjectUrl(PHOTOER_APPLY_BUCKET_NAME, minioService.cutFilename(PHOTOER_APPLY_BUCKET_NAME, dto.getIdCardBack()), 5, TimeUnit.MINUTES)); // 申请人证件背面
        }
        if (StringUtils.hasText(dto.getBodyPhoto())) {
            dto.setBodyPhoto(minioService.getPresignedObjectUrl(PHOTOER_APPLY_BUCKET_NAME, minioService.cutFilename(PHOTOER_APPLY_BUCKET_NAME, dto.getBodyPhoto()), 5, TimeUnit.MINUTES)); // 申请人半身照
        }
        if (StringUtils.hasText(dto.getSampleReels())) {
            dto.setSampleReels(minioService.getPresignedObjectUrl(PHOTOER_APPLY_BUCKET_NAME, minioService.cutFilename(PHOTOER_APPLY_BUCKET_NAME, dto.getSampleReels()), 5, TimeUnit.MINUTES)); // 作品集
        }
        return dto;
    }

    @Transactional
    public void photoerSave(PhotoerApplySaveRequest dto, String accountId) {
        Optional<PhotoerAudit> optional = findByAccountId(accountId);

        if (!optional.isPresent()) {
            PhotoerAudit _entity = PhotoerAuditMapStruct.INSTANCES.toEntity(dto);
            _entity.setAccountId(accountId);
            _entity.setIdCardFront(minioService.cutPath(photoerCopyFilename(dto.getIdCardFront(), "申请人身份证正面", accountId))); // 申请人身份证正面
            _entity.setIdCardBack(minioService.cutPath(photoerCopyFilename(dto.getIdCardBack(), "申请人身份证背面", accountId))); // 申请人身份证背面
            _entity.setBodyPhoto(minioService.cutPath(photoerCopyFilename(dto.getBodyPhoto(), "申请人半身照", accountId))); // 申请人半身照
            _entity.setSampleReels(minioService.cutPath(photoerCopyFilename(dto.getSampleReels(), "作品集", accountId))); // 作品集
            _entity.setAuditStatus(PhotoerAudit.AuditStatus.WAIT); // 审核状态（1：审核通过、0：未审核、-1:审核失败）
            photoerAuditMapper.insert(_entity);
        }
    }

    @Transactional
    public void photoerUpdate(PhotoerApplySaveRequest dto, String accountId) {
        Optional<PhotoerAudit> optional = findByAccountId(accountId);

        if (optional.isPresent()) {
            PhotoerAudit _entity = optional.get();
            /* 如果是审核通过了，不允许更改资料 */
            if (PhotoerAudit.AuditStatus.PASS == _entity.getAuditStatus()) {
                return;
            }

            PhotoerAudit entity = PhotoerAuditMapStruct.INSTANCES.toEntity(dto);
            entity.setId(_entity.getId());
            entity.setIdCardFront(minioService.cutPath(photoerCopyFilename(dto.getIdCardFront(), "申请人身份证正面", accountId))); // 申请人身份证正面
            entity.setIdCardBack(minioService.cutPath(photoerCopyFilename(dto.getIdCardBack(), "申请人身份证背面", accountId))); // 申请人身份证背面
            entity.setBodyPhoto(minioService.cutPath(photoerCopyFilename(dto.getBodyPhoto(), "申请人半身照", accountId))); // 申请人半身照
            entity.setSampleReels(minioService.cutPath(photoerCopyFilename(dto.getSampleReels(), "申请人半身照", accountId))); // 作品集
            photoerAuditMapper.updateById(entity);
        }
    }

    private String photoerCopyFilename(String fullPath, String text, String accountId) {
        /* 1. 文件复制到指定业务目录 */
        String relativePath = minioService.cutPath(fullPath);
        String bucketName = minioService.cutBucketName(relativePath);
        String filename = minioService.cutFilename(bucketName, relativePath);
        String newBucketName = PHOTOER_APPLY_BUCKET_NAME;
        String newFilename = String.format("%s/%s", accountId, filename);

        if (!minioService.isExists(bucketName, filename)) {
            throw new BusinessException(String.format("%s 文件不存在，请重新上传", text));
        }

        minioService.copyObject(bucketName, filename, newBucketName, newFilename);

        /* 2. 删除临时目录的文件 */
        if (minioService.isExists(newBucketName, newFilename)) {
            minioService.removeObject(bucketName, filename);
        }

        return minioService.getPresignedObjectUrl(newBucketName, newFilename, 5, TimeUnit.MINUTES);
    }

}
