package com.studio.nx.yxl.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.studio.nx.framework.dto.resp.BasePageResponse;
import com.studio.nx.framework.error.exception.BusinessException;
import com.studio.nx.uiac.entity.AccountAuths;
import com.studio.nx.uiac.service.AccountAuthsService;
import com.studio.nx.yxl.controller.admin.request.WithdrawalRecordVerifyRequest;
import com.studio.nx.yxl.controller.admin.search.WithdrawalRecordSearch;
import com.studio.nx.yxl.controller.app.request.my.withdrawal.WithdrawalRecordApplyRequest;
import com.studio.nx.yxl.dto.WithdrawalRecordDto;
import com.studio.nx.yxl.entity.IntegralCashRecord;
import com.studio.nx.yxl.entity.PhotoerAudit;
import com.studio.nx.yxl.entity.WithdrawalRecord;
import com.studio.nx.yxl.mapper.WithdrawalRecordMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import javax.transaction.Transactional;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Optional;

/**
 * @author Loki
 * @date 2021/6/1 15:24
 */
@Slf4j
@Service
public class WithdrawalRecordService {

    @Autowired
    private WithdrawalRecordMapper mapper;

    @Autowired
    private PhotoerAuditService photoerAuditService;

    @Autowired
    private IntegralCashRecordService integralCashRecordService;

    @Autowired
    private AccountAuthsService accountAuthsService;

    public WithdrawalRecordDto findByInfo(String id) {
        WithdrawalRecordDto dto = mapper.findByInfo(id);
        dto.setAvailableValue(integralCashRecordService.countMyCash(dto.getAccountId())); // 剩余可提现金额
        return dto;
    }

    public Optional<WithdrawalRecord> findById(String id) {
        return Optional.ofNullable(mapper.selectById(id));
    }

    public BasePageResponse<WithdrawalRecordDto> adminPage(WithdrawalRecordSearch dto, Integer page, Integer size) {
        IPage<WithdrawalRecordDto> p = mapper.adminPage(dto, new Page(page, size));

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    public BasePageResponse<WithdrawalRecord> appPage(String accountId, Integer page, Integer size) {
        Page<WithdrawalRecord> p = mapper.selectPage(new Page(page, size), new LambdaQueryWrapper<WithdrawalRecord>()
                .eq(WithdrawalRecord::getAccountId, accountId)
                .orderByDesc(WithdrawalRecord::getCreateAt)
        );

        return new BasePageResponse().setTotal(p.getTotal()).setData(p.getRecords());
    }

    @Transactional
    public void handleAudit(String id, WithdrawalRecordVerifyRequest request, String accountId) {
        Optional<WithdrawalRecord> optional = findById(id);
        if (!optional.isPresent()) {
            return;
        }

        WithdrawalRecord _withdrawalRecord = optional.get();
        if (WithdrawalRecord.AuditStatus.PASS == _withdrawalRecord.getAuditStatus()) {
            return;
        }

        if (WithdrawalRecord.AuditStatus.PASS == request.getAuditStatus()) {
            Assert.isTrue(StringUtils.hasText(request.getTransactionId()), "第三方支付标识为必填");

            /* 判断账号可提现的金额 */
            // todo 统一异常没有返回message
//            Assert.isTrue(integralCashRecordService.countMyCash(_withdrawalRecord.getAccountId()).compareTo(_withdrawalRecord.getValue()) != -1, "可提现金额不够");

            /* 判断账号可提现的金额 */
            if (integralCashRecordService.countMyCash(_withdrawalRecord.getAccountId()).compareTo(_withdrawalRecord.getValue()) == -1) {
                throw new BusinessException("可提现金额不够");
            }

            /* 扣减账号的现金 */
            integralCashRecordService.savePayCash(_withdrawalRecord.getAccountId(), IntegralCashRecord.Scene.WITHDRAWAL_CASH, "提现现金", _withdrawalRecord.getValue().negate());
        }

        _withdrawalRecord.setAuditStatus(request.getAuditStatus());
        _withdrawalRecord.setAuditMsg(request.getAuditMsg());
        _withdrawalRecord.setAuditAccountId(accountId);
        _withdrawalRecord.setAuditDate(LocalDateTime.now());
        _withdrawalRecord.setTransactionId(request.getTransactionId());
        mapper.updateById(_withdrawalRecord);
    }

    @Transactional
    public void apply(WithdrawalRecordApplyRequest request, String accountId) {
        /* 获取摄影师认证记录 */
        Optional<PhotoerAudit> photoerAuditOptional = photoerAuditService.findByAccountId(accountId);
        if (!photoerAuditOptional.isPresent()) {
            throw new BusinessException("账号还没提交摄影师认证");
        }

        PhotoerAudit photoerAudit = photoerAuditOptional.get();
        if (PhotoerAudit.AuditStatus.PASS != photoerAudit.getAuditStatus()) {
            throw new BusinessException("账号还没审核通过摄影师认证");
        }

        /* 判断当前账号是否还有待审核的提现申请。 */
        if (countWait(accountId) > 0) {
            throw new BusinessException("您的提现申请失败，原因：还有待审批的提现。");
        }
        if (request.getValue().compareTo(new BigDecimal("0.01")) == -1) {
            throw new BusinessException("提现最少额度为¥0.01元");
        }

        /* 判断如果是提现到微信到，需要判断是否已绑定微信 */
        String channels = ""; // 渠道
        String transactionAccountId = ""; // 第三方平台标识
        switch (request.getChannels()) {
            case 0:
                channels = "微信";
                Optional<AccountAuths> accountAuthsWeixinOpenAppOpenOptional = accountAuthsService.findByAccountId(AccountAuths.IdentityType.WEIXIN_OPEN_APP_OPEN, accountId);
                if (!accountAuthsWeixinOpenAppOpenOptional.isPresent()) {
                    throw new BusinessException("您还没绑定微信");
                }
                transactionAccountId = accountAuthsWeixinOpenAppOpenOptional.get().getIdentifier();
                break;
            case 1:
                channels = "支付宝";
                transactionAccountId = "支付宝账号"; // todo
                break;
            case 2:
                channels = "银联";
                transactionAccountId = photoerAudit.getBankCardNumber();
                break;
        }

        /* 判断账号可提现的金额 */
        if (integralCashRecordService.countMyCash(accountId).compareTo(request.getValue()) == -1) {
            throw new BusinessException("可提现金额不够");
        }

        BigDecimal value = request.getValue(); // 数值

        WithdrawalRecord _entity = new WithdrawalRecord();
        _entity.setAccountId(accountId); // 账号标识
        _entity.setPhonenumber(photoerAudit.getPhonenumber()); // 申请人手机号码
        _entity.setRealName(photoerAudit.getRealName()); // 真实姓名
        _entity.setTransactionAccountId(transactionAccountId); // 第三方平台标识
        _entity.setChannels(channels);// 渠道（微信、支付宝、银行卡）
        _entity.setValue(value); // 数值（可正可负）
        _entity.setAuditStatus(WithdrawalRecord.AuditStatus.WAIT); // 审核状态

        mapper.insert(_entity);
    }

    private Integer countWait(String accountId) {
        return mapper.selectCount(new LambdaQueryWrapper<WithdrawalRecord>()
                .eq(WithdrawalRecord::getAccountId, accountId)
                .eq(WithdrawalRecord::getAuditStatus, WithdrawalRecord.AuditStatus.WAIT)
        );
    }

}
